from policyengine_us.model_api import *


class il_hbi_immigration_status_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Has eligible immigration status for Illinois HBI"
    definition_period = YEAR
    defined_for = StateCode.IL
    reference = (
        "https://hfs.illinois.gov/medicalclients/healthbenefitsforimmigrants.html",
        "https://www.dhs.state.il.us/page.aspx?item=161600",
    )
    # Illinois Health Benefits for Immigrants (HBI) covers residents who are not
    # eligible for federal Medicaid due to their immigration status.

    def formula(person, period, parameters):
        # Eligible for HBI if NOT eligible for federal Medicaid
        # due to immigration status
        federal_eligible = person(
            "is_medicaid_immigration_status_eligible", period
        )
        return ~federal_eligible



from policyengine_us.model_api import *


class ks_tanf_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = (
        "Eligible for Kansas Temporary Assistance for Needy Families (TANF)"
    )
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/kansas/K-A-R-30-4-70",
        "https://www.law.cornell.edu/regulations/kansas/K-A-R-30-4-50",
        "https://ksrevisor.gov/statutes/chapters/ch39/039_007_0009.html",
    )
    defined_for = StateCode.KS

    def formula(spm_unit, period, parameters):
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        immigration_eligible = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )
        gross_income_eligible = spm_unit(
            "ks_tanf_gross_income_eligible", period
        )
        net_income_eligible = spm_unit("ks_tanf_income_eligible", period)
        resources_eligible = spm_unit("ks_tanf_resources_eligible", period)
        return (
            demographic_eligible
            & immigration_eligible
            & gross_income_eligible
            & net_income_eligible
            & resources_eligible
        )



from policyengine_us.model_api import *


class ak_atap_net_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Alaska ATAP net income eligible"
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/alaska/7-AAC-45.470"
    defined_for = StateCode.AK

    def formula(spm_unit, period, parameters):
        # Test 2: Countable income (after deductions) <= need standard
        countable_income = spm_unit("ak_atap_countable_income", period)
        need_standard = spm_unit("ak_atap_need_standard", period)
        return countable_income <= need_standard



from policyengine_us.model_api import *


class pell_grant_head_contribution(Variable):
    value_type = float
    entity = Person
    unit = USD
    label = "Pell Grant head contribution"
    definition_period = YEAR

    def formula(person, period, parameters):
        available_income = person("pell_grant_head_available_income", period)
        assets = person("pell_grant_contribution_from_assets", period)
        adjusted_available_income = available_income + assets
        formula = person("pell_grant_formula", period)
        p = parameters(period).gov.ed.pell_grant
        positive_head_contribution = p.head.marginal_rate.calc(
            adjusted_available_income
        )
        negative_head_contribution = max_(
            adjusted_available_income * p.head.negative_rate,
            p.head.min_contribution,
        )
        total_head_contribution = where(
            adjusted_available_income >= 0,
            positive_head_contribution,
            negative_head_contribution,
        )
        total = where(
            formula == formula.possible_values.B,
            adjusted_available_income,
            total_head_contribution,
        )

        if p.uses_sai:
            return total
        else:
            # EFC divides by number of dependents in college
            dependents = person.tax_unit(
                "pell_grant_dependents_in_college", period
            )
            amount_per_dependent = np.zeros_like(total)
            mask = dependents > 0
            amount_per_dependent[mask] = total[mask] / dependents[mask]
            return amount_per_dependent



from policyengine_us.model_api import *
from policyengine_core.periods import period as period_
from policyengine_core.periods import instant


def create_ri_social_security_exemption() -> Reform:
    class ri_social_security_modification_eligible(Variable):
        value_type = bool
        entity = TaxUnit
        label = "Eligible for the Rhode Island Social Security Modification"
        definition_period = YEAR
        reference = "https://tax.ri.gov/sites/g/files/xkgbur541/files/2022-12/Social%20Security%20Worksheet_w.pdf"
        defined_for = StateCode.RI

        def formula(tax_unit, period, parameters):
            p_reform = parameters(
                period
            ).gov.contrib.states.ri.social_security_exemption
            p_baseline = parameters(
                period
            ).gov.states.ri.tax.income.agi.subtractions.social_security.limit

            # Get inputs
            income = tax_unit("adjusted_gross_income", period)
            filing_status = tax_unit("filing_status", period)
            birth_year = tax_unit("older_spouse_birth_year", period)

            # Baseline eligibility: age + income requirements
            baseline_age_eligible = birth_year <= p_baseline.birth_year
            baseline_income_limit = p_baseline.income[filing_status]
            baseline_income_eligible = income < baseline_income_limit
            baseline_eligible = (
                baseline_age_eligible & baseline_income_eligible
            )

            # Reform eligibility depends on which phase:
            # - income_limit_applies=True: universal exemption (2029+)
            # - age_requirement_applies=False: no age requirement (2027+)
            reform_income_limit = p_reform.income_limit[filing_status]
            reform_income_eligible = income < reform_income_limit
            reform_age_eligible = where(
                p_reform.age_requirement_applies,
                birth_year <= p_baseline.birth_year,
                True,
            )
            reform_eligible_with_limits = (
                reform_age_eligible & reform_income_eligible
            )

            # Universal exemption when income limit is repealed
            reform_eligible = where(
                p_reform.income_limit_applies,
                True,
                reform_eligible_with_limits,
            )

            # Select based on whether reform is in effect
            return where(
                p_reform.in_effect, reform_eligible, baseline_eligible
            )

    class ri_social_security_modification(Variable):
        value_type = float
        entity = TaxUnit
        label = "Rhode Island Social Security Modification"
        unit = USD
        definition_period = YEAR
        reference = "https://tax.ri.gov/sites/g/files/xkgbur541/files/2022-12/Social%20Security%20Worksheet_w.pdf"
        defined_for = "ri_social_security_modification_eligible"

        def formula(tax_unit, period, parameters):
            p_reform = parameters(
                period
            ).gov.contrib.states.ri.social_security_exemption
            p_baseline = parameters(
                period
            ).gov.states.ri.tax.income.agi.subtractions.social_security.limit

            person = tax_unit.members
            head_or_spouse = person("is_tax_unit_head_or_spouse", period)
            birth_year = person("birth_year", period)
            taxable_social_security = person("taxable_social_security", period)
            total_social_security = person("social_security", period)

            # Calculate total head/spouse taxable SS (used in reform)
            head_or_spouse_taxable_ss = tax_unit.sum(
                taxable_social_security * head_or_spouse
            )

            # Calculate aged portion (used in baseline and when age req applies)
            aged = birth_year <= p_baseline.birth_year
            head_or_spouse_aged = head_or_spouse & aged

            aged_head_or_spouse_ss = tax_unit.sum(
                total_social_security * head_or_spouse_aged
            )
            head_or_spouse_ss = tax_unit.sum(
                total_social_security * head_or_spouse
            )

            aged_ss_ratio = np.zeros_like(head_or_spouse_ss)
            mask = head_or_spouse_ss != 0
            aged_ss_ratio[mask] = (
                aged_head_or_spouse_ss[mask] / head_or_spouse_ss[mask]
            )

            baseline_amount = head_or_spouse_taxable_ss * aged_ss_ratio

            # Reform amount: all taxable SS when age requirement removed
            reform_amount = where(
                p_reform.age_requirement_applies,
                baseline_amount,
                head_or_spouse_taxable_ss,
            )

            return where(p_reform.in_effect, reform_amount, baseline_amount)

    class reform(Reform):
        def apply(self):
            self.update_variable(ri_social_security_modification_eligible)
            self.update_variable(ri_social_security_modification)

    return reform


def create_ri_social_security_exemption_reform(
    parameters, period, bypass: bool = False
):
    if bypass:
        return create_ri_social_security_exemption()

    p = parameters.gov.contrib.states.ri.social_security_exemption

    reform_active = False
    current_period = period_(period)

    for i in range(5):
        if p(current_period).in_effect:
            reform_active = True
            break
        current_period = current_period.offset(1, "year")

    if reform_active:
        return create_ri_social_security_exemption()
    else:
        return None


ri_social_security_exemption = create_ri_social_security_exemption_reform(
    None, None, bypass=True
)



from policyengine_us.model_api import *


class co_omnisalud_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Eligible for Colorado OmniSalud"
    definition_period = YEAR
    defined_for = StateCode.CO
    reference = [
        "https://connectforhealthco.com/get-started/omnisalud/",
        "https://coloradoimmigrant.org/wp-content/uploads/2024/03/Eng.-OmniSalud-Guide-2024.pdf",
    ]
    documentation = """
    Colorado OmniSalud provides marketplace coverage with premium subsidies
    to individuals who are ineligible for Health First Colorado (Medicaid),
    CHP+, or federal ACA subsidies due to their immigration status.
    """

    def formula(person, period, parameters):
        p = parameters(period).gov.states.co.hcpf.omnisalud.eligibility

        # Program must be in effect
        in_effect = p.in_effect

        # Must have eligible immigration status (undocumented, DACA before 2025)
        immigration_eligible = person(
            "co_omnisalud_immigration_status_eligible", period
        )

        # Must meet income requirements for financial assistance
        income_eligible = person("co_omnisalud_income_eligible", period)

        # Must not be eligible for other coverage
        # (Medicaid/CHP+ eligibility already checked via immigration status)

        return in_effect & immigration_eligible & income_eligible



from policyengine_us.model_api import *


class ok_tanf_eid_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Oklahoma TANF earned income disregard"
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/oklahoma/OAC-340-10-3-31.1"
    )
    defined_for = StateCode.OK

    def formula(spm_unit, period, parameters):
        # Per OAC 340:10-3-31.1(a): EID applies when gross earned income <= $2,064
        gross_earned = add(spm_unit, period, ["tanf_gross_earned_income"])
        p = parameters(period).gov.states.ok.dhs.tanf.income
        return gross_earned <= p.disregard.cap



from policyengine_us.model_api import *


class ri_works(Variable):
    value_type = float
    entity = SPMUnit
    label = "Rhode Island Works benefit"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://rules.sos.ri.gov/Regulations/part/218-20-00-2",
        "https://dhs.ri.gov/programs-and-services/ri-works-program",
    )
    defined_for = "ri_works_eligible"

    def formula(spm_unit, period, parameters):
        payment_standard = spm_unit("ri_works_payment_standard", period)
        countable_income = spm_unit("ri_works_countable_income", period)
        return max_(payment_standard - countable_income, 0)



from policyengine_us.model_api import *


class or_tanf_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Oregon TANF income eligible"
    definition_period = MONTH
    reference = "https://oregon.public.law/rules/oar_461-160-0100"
    defined_for = StateCode.OR

    def formula(spm_unit, period, parameters):
        # Standard test: countable < limit AND adjusted < limit
        passes_countable_test = spm_unit(
            "or_tanf_countable_income_eligible", period
        )
        passes_adjusted_test = spm_unit(
            "or_tanf_adjusted_income_eligible", period
        )
        passes_standard_test = passes_countable_test & passes_adjusted_test

        # ELI test: countable < (payment_standard * multiplier)
        passes_eli_test = spm_unit("or_tanf_eli_income_eligible", period)

        eli_eligible = spm_unit("or_tanf_eli_eligible", period)
        return where(eli_eligible, passes_eli_test, passes_standard_test)



from policyengine_us.model_api import *


class in_tanf_countable_income_for_eligibility(Variable):
    value_type = float
    entity = SPMUnit
    label = "Indiana TANF countable income for eligibility determination"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://iar.iga.in.gov/code/2026/470/10.3#470-10.3-4-4",
        "https://iga.in.gov/laws/2025/ic/titles/12/#12-14-2-3",
    )
    defined_for = StateCode.IN

    adds = [
        "in_tanf_countable_earned_income_for_eligibility",
        "tanf_gross_unearned_income",
    ]



from policyengine_us.model_api import *


class in_tanf_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Indiana TANF countable earned income"
    unit = USD
    definition_period = MONTH
    reference = "https://iar.iga.in.gov/code/2026/470/10.3#470-10.3-4-4"
    defined_for = StateCode.IN

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states["in"].fssa.tanf.income.deductions
        person = spm_unit.members
        gross_earned = person("tanf_gross_earned_income", period)
        # 75% disregard per 470 IAC 10.3-4-4(d)(1)
        return spm_unit.sum(
            gross_earned * (1 - p.benefit.earned_income_disregard.rate)
        )



from policyengine_us.model_api import *


class ks_tanf_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Kansas TANF income eligibility"
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/kansas/K-A-R-30-4-110",
        "https://content.dcf.ks.gov/ees/keesm/current/keesm7110.htm",
    )
    defined_for = StateCode.KS

    def formula(spm_unit, period, parameters):
        # Per K.A.R. 30-4-110 and KEESM 7110:
        # Countable income must be less than the payment standard
        countable_income = spm_unit("ks_tanf_countable_income", period)
        payment_standard = spm_unit("ks_tanf_maximum_benefit", period)
        return countable_income < payment_standard



from policyengine_us.model_api import *


class sd_tanf_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "South Dakota TANF countable earned income"
    unit = USD
    definition_period = MONTH
    reference = "https://sdlegislature.gov/Rules/Administrative/67:10:03:05"
    defined_for = StateCode.SD

    adds = ["sd_tanf_countable_earned_income_person"]



from policyengine_us.model_api import *


class nj_wfnj_payment_levels(Variable):
    value_type = float
    entity = SPMUnit
    label = "New Jersey WFNJ payment levels"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.NJ
    reference = (
        "https://www.law.cornell.edu/regulations/new-jersey/N-J-A-C-10-90-3-3"
    )

    def formula(spm_unit, period, parameters):
        size = spm_unit("spm_unit_size", period.this_year)
        p = parameters(period).gov.states.nj.njdhs.wfnj
        capped_size = min_(size, p.max_household_size)
        additional = size - capped_size
        base = p.payment_levels.amount[capped_size]
        additional_amount = p.payment_levels.additional_person * additional
        return base + additional_amount



from policyengine_us.model_api import *


class mo_tanf(Variable):
    value_type = float
    entity = SPMUnit
    label = "Missouri Temporary Assistance for Needy Families (TANF)"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/missouri/13-CSR-40-2-310",
        "https://dssmanuals.mo.gov/temporary-assistance-case-management/0210-010-15/",
    )
    defined_for = "mo_tanf_eligible"

    def formula(spm_unit, period, parameters):
        maximum_benefit = spm_unit("mo_tanf_maximum_benefit", period)
        countable_income = spm_unit("mo_tanf_countable_income", period)
        benefit = max_(maximum_benefit - countable_income, 0)
        p = parameters(period).gov.states.mo.dss.tanf
        return where(benefit >= p.minimum_payment, benefit, 0)



from policyengine_us.model_api import *


class me_tanf_countable_unearned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Maine TANF countable unearned income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.mainelegislature.org/legis/statutes/22/title22sec3762.html",
        "https://www.law.cornell.edu/regulations/maine/10-144-C-M-R-ch-331",
    )
    defined_for = StateCode.ME

    def formula(spm_unit, period, parameters):
        # Per 22 M.R.S. Section 3762: First $50/month of child support
        # is excluded from income calculations
        p = parameters(period).gov.states.me.dhhs.tanf

        gross_unearned = add(spm_unit, period, ["tanf_gross_unearned_income"])
        child_support = add(spm_unit, period, ["child_support_received"])
        child_support_deduction = min_(
            child_support, p.child_support_deduction
        )

        return max_(gross_unearned - child_support_deduction, 0)



from policyengine_us.model_api import *


class in_tanf_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Indiana TANF eligible"
    definition_period = MONTH
    reference = "https://iar.iga.in.gov/code/2026/470/10.3"  # 470 IAC 10.3
    defined_for = StateCode.IN

    def formula(spm_unit, period, parameters):
        # Demographic eligibility - use federal baseline
        # Per IC 12-14-1-0.5 and 470 IAC 10.3-3
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)

        # Immigration eligibility - must have at least one citizen or qualified noncitizen
        # Per IC 12-14-1-1
        person = spm_unit.members
        immigration_eligible = spm_unit.any(
            person("is_citizen_or_legal_immigrant", period.this_year)
        )

        # Income eligibility - state-specific
        income_eligible = spm_unit("in_tanf_income_eligible", period)

        # Resources eligibility - state-specific
        # Per 470 IAC 10.3-3-6
        resources_eligible = spm_unit("in_tanf_resources_eligible", period)

        return (
            demographic_eligible
            & immigration_eligible
            & income_eligible
            & resources_eligible
        )



from policyengine_us.model_api import *


class wi_works(Variable):
    value_type = float
    entity = SPMUnit
    label = "Wisconsin Works"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://docs.legis.wisconsin.gov/statutes/statutes/49/iii/148",
        "https://dcf.wisconsin.gov/manuals/w-2-manual/Production/07/7.4.1_Community_Service_Jobs_(CSJ).htm",
    )
    defined_for = "wi_works_eligible"

    def formula(spm_unit, period, parameters):
        placement = spm_unit("wi_works_placement", period)
        p = parameters(period).gov.states.wi.dcf.works.placement
        return p.amount[placement]



from policyengine_us.model_api import *
from policyengine_us.variables.gov.hhs.tax_unit_fpg import fpg


class il_ihwap_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Illinois IHWAP income eligible"
    definition_period = YEAR
    defined_for = StateCode.IL
    reference = (
        "https://www.law.cornell.edu/uscode/text/42/6862#7",
        "https://dceo.illinois.gov/communityservices/homeweatherization.html",
    )

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.il.dceo.ihwap.eligibility
        p_hhs = parameters(period).gov.hhs
        income = add(spm_unit, period, ["irs_gross_income"])

        # IL IHWAP Program Year N uses FPL from year N-1 and SMI from (N-1)-10-01
        prior_year = str(period.start.year - 1)

        # 200% FPL threshold (all funding sources)
        size = spm_unit("spm_unit_size", period)
        state_group = spm_unit.household("state_group_str", period)
        fpg_amount = fpg(size, state_group, prior_year, parameters)
        fpg_limit = fpg_amount * p.fpg_limit

        # 60% SMI threshold (State funds only) per HHS LIHEAP-IM-2025-02
        # Only applies to households larger than smi_threshold_size
        smi = spm_unit("il_ihwap_hhs_smi", period)
        smi_limit = smi * p_hhs.liheap.smi_limit
        # Sizes 1-7: 200% FPL only
        # Sizes 8+: max(200% FPL, 60% SMI)
        size = spm_unit("spm_unit_size", period)
        large_household_limit = max_(fpg_limit, smi_limit)
        income_limit = where(
            size > p.smi_threshold_size,
            large_household_limit,
            fpg_limit,
        )
        return income <= income_limit



from policyengine_us.model_api import *


class ga_tanf_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Georgia TANF due to income"
    definition_period = MONTH
    reference = (
        "https://pamms.dhs.ga.gov/dfcs/tanf/1525/",
        "https://pamms.dhs.ga.gov/dfcs/tanf/appendix-a/",
    )
    defined_for = StateCode.GA

    def formula(spm_unit, period, parameters):
        # Gross income test - use federal TANF variables
        gross_income = add(
            spm_unit,
            period,
            ["tanf_gross_earned_income", "tanf_gross_unearned_income"],
        )

        gross_income_ceiling = spm_unit("ga_tanf_gross_income_ceiling", period)
        passes_gross_test = gross_income <= gross_income_ceiling

        # Net income test
        countable_income = spm_unit("ga_tanf_countable_income", period)
        standard_of_need = spm_unit("ga_tanf_standard_of_need", period)
        passes_net_test = countable_income < standard_of_need

        return passes_gross_test & passes_net_test



from policyengine_us.model_api import *


class nv_tanf_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Nevada TANF eligible"
    definition_period = MONTH
    reference = "https://dss.nv.gov/TANF/TANF_FAQ/"
    defined_for = StateCode.NV

    def formula(spm_unit, period, parameters):
        # Use federal demographic eligibility (minor child with deprived parent)
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)

        # Must have at least one U.S. citizen or qualified immigrant
        immigration_eligible = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )

        # Must meet income eligibility
        income_eligible = spm_unit("nv_tanf_income_eligible", period)

        # Must meet resource eligibility ($10,000 limit)
        resource_eligible = spm_unit("nv_tanf_resources_eligible", period)

        return (
            demographic_eligible
            & immigration_eligible
            & income_eligible
            & resource_eligible
        )



from policyengine_us.model_api import *


class nh_fanf(Variable):
    value_type = float
    entity = SPMUnit
    label = "New Hampshire Financial Assistance to Needy Families"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://gc.nh.gov/rsa/html/xii/167/167-77-g.htm",
        "https://www.dhhs.nh.gov/sr_htm/html/sr_97-03_dated_02_97.htm",
    )
    defined_for = "nh_fanf_eligible"

    def formula(spm_unit, period, parameters):
        payment_standard = spm_unit("nh_fanf_payment_standard", period)
        countable_income = spm_unit("nh_fanf_countable_income", period)
        return max_(payment_standard - countable_income, 0)



from policyengine_us.model_api import *


class nm_works_gross_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "New Mexico Works gross income"
    unit = USD
    definition_period = MONTH
    reference = "https://www.srca.nm.gov/parts/title08/08.102.0520.html"
    defined_for = StateCode.NM

    adds = ["tanf_gross_earned_income", "tanf_gross_unearned_income"]



from policyengine_us.model_api import *
from policyengine_core.periods import period as period_


def create_cdcc_single_parent_work_requirement() -> Reform:
    """
    Reform that changes CDCTC eligibility to require at least one parent to work
    instead of both parents.

    Current law: For married couples, childcare expenses are capped at the
    minimum of the two spouses' earnings, effectively requiring both to work.

    This reform: Changes the cap to the maximum of the two spouses' earnings,
    allowing families to qualify if at least one parent works.

    Note: We override cdcc_relevant_expenses directly (rather than
    min_head_spouse_earned) to avoid affecting state programs that also
    use min_head_spouse_earned.
    """

    class cdcc_relevant_expenses(Variable):
        value_type = float
        entity = TaxUnit
        label = "CDCC-relevant care expenses"
        unit = USD
        definition_period = YEAR
        reference = (
            "https://www.law.cornell.edu/uscode/text/26/21#c",
            "https://www.law.cornell.edu/uscode/text/26/21#d_1",
        )

        def formula(tax_unit, period, parameters):
            expenses = tax_unit("tax_unit_childcare_expenses", period)
            cdcc_limit = tax_unit("cdcc_limit", period)
            eligible_capped_expenses = min_(expenses, cdcc_limit)

            # Reform: Use max of head/spouse earnings instead of min
            # This allows eligibility if at least one parent works
            is_joint = tax_unit("tax_unit_is_joint", period)
            head_earnings = tax_unit("head_earned", period)
            spouse_earnings = tax_unit("spouse_earned", period)
            earnings_cap = where(
                is_joint, max_(head_earnings, spouse_earnings), head_earnings
            )

            return min_(eligible_capped_expenses, earnings_cap)

    class reform(Reform):
        def apply(self):
            self.update_variable(cdcc_relevant_expenses)

    return reform


def create_cdcc_single_parent_work_requirement_reform(
    parameters, period, bypass: bool = False
):
    if bypass:
        return create_cdcc_single_parent_work_requirement()

    p = parameters.gov.contrib.cdcc.single_parent_work_requirement

    # Check if reform is active in current period or next 5 years
    reform_active = False
    current_period = period_(period)

    for i in range(5):
        if p(current_period).in_effect:
            reform_active = True
            break
        current_period = current_period.offset(1, "year")

    if reform_active:
        return create_cdcc_single_parent_work_requirement()
    else:
        return None


cdcc_single_parent_work_requirement = (
    create_cdcc_single_parent_work_requirement_reform(None, None, bypass=True)
)



from policyengine_us.model_api import *


class il_scretd_eligible(Variable):
    value_type = bool
    entity = TaxUnit
    label = "Illinois Senior Citizens Real Estate Tax Deferral eligibility"
    definition_period = YEAR
    defined_for = StateCode.IL
    reference = "https://www.ilga.gov/legislation/ilcs/ilcs3.asp?ActID=1454&ChapterID=31"

    def formula(tax_unit, period, parameters):
        age_eligible = add(tax_unit, period, ["il_scretd_age_eligible"]) > 0
        income_eligible = tax_unit("il_scretd_income_eligible", period)
        return age_eligible & income_eligible



from policyengine_us.model_api import *


def create_ny_s04487_newborn_credit() -> Reform:
    """
    NY Senate Bill S04487 - Supplemental Empire State Child Tax Credit for Newborns

    Establishes a $1,000 supplemental credit for newborns (children born in
    the current or previous tax year who have not been previously claimed).

    Reference: https://www.nysenate.gov/legislation/bills/2025/S4487
    """

    class ny_s04487_qualifying_newborn(Variable):
        value_type = bool
        entity = Person
        label = "Qualifies as newborn for NY S04487 credit"
        definition_period = YEAR
        reference = "https://www.nysenate.gov/legislation/bills/2025/S4487"
        defined_for = StateCode.NY

        def formula(person, period, parameters):
            p = parameters(period).gov.contrib.states.ny.s04487
            in_effect = p.in_effect
            # Child must be age 0 or 1 (born in current or previous tax year)
            age = person("age", period)
            age_eligible = age <= p.max_age
            # Must be a dependent
            is_dependent = person("is_tax_unit_dependent", period)
            return in_effect & age_eligible & is_dependent

    class ny_s04487_newborn_count(Variable):
        value_type = int
        entity = TaxUnit
        label = "Number of qualifying newborns for NY S04487 credit"
        definition_period = YEAR
        reference = "https://www.nysenate.gov/legislation/bills/2025/S4487"
        defined_for = StateCode.NY

        def formula(tax_unit, period, parameters):
            person = tax_unit.members
            qualifying = person("ny_s04487_qualifying_newborn", period)
            return tax_unit.sum(qualifying)

    class ny_s04487_newborn_credit(Variable):
        value_type = float
        entity = TaxUnit
        label = "NY S04487 supplemental newborn credit"
        unit = USD
        definition_period = YEAR
        reference = "https://www.nysenate.gov/legislation/bills/2025/S4487"
        defined_for = StateCode.NY

        def formula(tax_unit, period, parameters):
            p = parameters(period).gov.contrib.states.ny.s04487
            in_effect = p.in_effect
            newborn_count = tax_unit("ny_s04487_newborn_count", period)
            credit_per_newborn = p.amount
            return where(in_effect, newborn_count * credit_per_newborn, 0)

    class ny_refundable_credits(Variable):
        value_type = float
        entity = TaxUnit
        label = "NY refundable credits"
        unit = USD
        definition_period = YEAR
        defined_for = StateCode.NY

        def formula(tax_unit, period, parameters):
            p = parameters(period).gov.states.ny.tax.income.credits
            standard_credits = add(tax_unit, period, p.refundable)
            # Add newborn credit from S04487
            newborn_credit = tax_unit("ny_s04487_newborn_credit", period)
            return standard_credits + newborn_credit

    class reform(Reform):
        def apply(self):
            self.update_variable(ny_s04487_qualifying_newborn)
            self.update_variable(ny_s04487_newborn_count)
            self.update_variable(ny_s04487_newborn_credit)
            self.update_variable(ny_refundable_credits)

    return reform


def create_ny_s04487_newborn_credit_reform(
    parameters, period, bypass: bool = False
):
    if bypass:
        return create_ny_s04487_newborn_credit()

    p = parameters(period).gov.contrib.states.ny.s04487

    if p.in_effect:
        return create_ny_s04487_newborn_credit()
    else:
        return None


ny_s04487_newborn_credit = create_ny_s04487_newborn_credit_reform(
    None, None, bypass=True
)



from policyengine_us.model_api import *


class or_tanf(Variable):
    value_type = float
    entity = SPMUnit
    label = "Oregon Temporary Assistance for Needy Families (TANF)"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://oregon.public.law/rules/oar_461-155-0030",
        "https://oregon.public.law/rules/oar_461-165-0060",
    )
    defined_for = "or_tanf_eligible"

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states["or"].odhs.tanf
        payment_standard = spm_unit("or_tanf_payment_standard", period)
        adjusted_income = spm_unit("or_tanf_adjusted_income", period)
        calculated_benefit = max_(payment_standard - adjusted_income, 0)
        return where(
            calculated_benefit >= p.minimum_benefit,
            calculated_benefit,
            0,
        )



from policyengine_us.model_api import *


class va_tanf_grant_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "VA TANF grant standard"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.VA
    reference = "https://www.dss.virginia.gov/files/division/bp/tanf/manual/300_11-20.pdf#page=47"

    def formula(spm_unit, period, parameters):
        unit_size = spm_unit("spm_unit_size", period.this_year)
        p = parameters(period).gov.states.va.dss.tanf
        # Access July 2020 base values
        p_grant_standard = (
            parameters.gov.states.va.dss.tanf.payment.grant_standard(
                f"2020-10-01"
            )
        )
        ceiling = min_(unit_size, p.max_unit_size)
        additional = unit_size - ceiling

        county = spm_unit.household("county_str", period)
        if_group3 = np.isin(county, p.localities.group3)
        main = where(
            if_group3,
            p_grant_standard.group3.main[ceiling],
            p_grant_standard.group2.main[ceiling],
        )
        addition = where(
            if_group3,
            p_grant_standard.group3.addition,
            p_grant_standard.group2.addition,
        )

        base_amount = main + additional * addition
        # Apply standard multiplier for increases since July 2020 base

        return base_amount * p.standard_multiplier



from policyengine_us.model_api import *


class ne_adc_resources_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Nebraska ADC resources eligible"
    definition_period = YEAR
    reference = (
        "https://nebraskalegislature.gov/laws/statutes.php?statute=68-1726",
        "https://nebraskalegislature.gov/laws/statutes.php?statute=68-1713",
    )
    defined_for = StateCode.NE

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ne.dhhs.adc
        # spm_unit_assets is a YEAR variable
        countable_resources = spm_unit("spm_unit_assets", period)
        size = spm_unit("spm_unit_size", period)
        # Resource limits: $4,000 for 1 person, $6,000 for 2+ persons
        limit = p.resources.limit.calc(size)
        return countable_resources <= limit



from policyengine_us.model_api import *


class nm_works_gross_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "New Mexico Works gross income eligible"
    definition_period = MONTH
    reference = "https://www.srca.nm.gov/parts/title08/08.102.0520.html"
    defined_for = StateCode.NM

    def formula(spm_unit, period, parameters):
        # Per 8.102.500.8(B) NMAC, gross income must be under 85% FPL
        p = parameters(period).gov.states.nm.hca.nm_works.income.gross_limit
        gross_income = spm_unit("nm_works_gross_income", period)
        fpg = spm_unit("tanf_fpg", period)
        return gross_income <= fpg * p.rate



from policyengine_us.model_api import *


class pa_tanf_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Pennsylvania TANF eligibility"
    definition_period = MONTH
    defined_for = StateCode.PA
    reference = (
        "https://www.pa.gov/agencies/dhs/resources/cash-assistance/tanf"
    )

    def formula(spm_unit, period, parameters):
        person = spm_unit.members

        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        has_citizen = spm_unit.any(
            person("is_citizen_or_legal_immigrant", period)
        )
        income_eligible = spm_unit("pa_tanf_income_eligible", period)
        resources_eligible = spm_unit("pa_tanf_resources_eligible", period)

        return (
            demographic_eligible
            & has_citizen
            & income_eligible
            & resources_eligible
        )



from policyengine_us.model_api import *


class mn_mfip_countable_unearned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Minnesota MFIP countable unearned income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.revisor.mn.gov/statutes/cite/256P.06#stat.256P.06.3"
    )
    defined_for = StateCode.MN
    # Per MN Stat. 256P.06, Subd. 3:
    # Gross unearned income minus child support exclusion (up to $100/1 child, $200/2+).
    adds = ["tanf_gross_unearned_income"]
    subtracts = ["mn_mfip_child_support_income_exclusion"]



from policyengine_us.model_api import *


class id_tafi_grant_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Idaho TAFI grant standard"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/idaho/IDAPA-16.03.08.249",
        "https://www.law.cornell.edu/regulations/idaho/IDAPA-16.03.08.250",
        "https://www.law.cornell.edu/regulations/idaho/IDAPA-16.03.08.252",
    )
    defined_for = StateCode.ID

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.id.tafi
        countable_earned = spm_unit("id_tafi_countable_earned_income", period)
        countable_unearned = spm_unit(
            "id_tafi_countable_unearned_income", period
        )
        work_incentive = spm_unit("id_tafi_work_incentive_amount", period)

        # With earned: work_incentive - earned; Without: maximum_grant
        base = where(
            countable_earned > 0,
            work_incentive - countable_earned,
            p.maximum_grant,
        )
        return max_(base - countable_unearned, 0)



from policyengine_us.model_api import *


class ak_atap_childcare_deduction(Variable):
    value_type = float
    entity = SPMUnit
    label = "Alaska ATAP childcare deduction"
    unit = USD
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/alaska/7-AAC-45.485"
    defined_for = StateCode.AK

    def formula(spm_unit, period, parameters):
        # Per 7 AAC 45.485: Child care expenses excluded up to:
        # - $200/month per child under age 2
        # - $175/month per child age 2 and older
        p = parameters(period).gov.states.ak.dpa.atap.income.deductions
        person = spm_unit.members
        dependent = person("is_tax_unit_dependent", period)
        age = person("age", period.this_year)
        childcare_expenses = spm_unit("childcare_expenses", period)
        max_per_dependent = p.childcare.calc(age) * dependent
        total_max_disregard = spm_unit.sum(max_per_dependent)
        return min_(childcare_expenses, total_max_disregard)



from policyengine_us.model_api import *


class pa_tanf_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Pennsylvania TANF countable income"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.PA
    reference = "https://www.pacodeandbulletin.gov/Display/pacode?file=/secure/pacode/data/055/chapter183/chap183toc.html"

    adds = [
        "pa_tanf_countable_earned_income",
        "tanf_gross_unearned_income",
    ]



from policyengine_us.model_api import *


class sd_tanf_countable_unearned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "South Dakota TANF countable unearned income"
    unit = USD
    definition_period = MONTH
    reference = "https://sdlegislature.gov/Rules/Administrative/67:10:03:04"
    defined_for = StateCode.SD

    # NOTE: Result will never go below 0 because child_support_received
    # is already included in tanf_gross_unearned_income.
    adds = ["tanf_gross_unearned_income"]
    subtracts = ["child_support_received"]



from policyengine_us.model_api import *


class il_hbi_senior_income_level(Variable):
    value_type = float
    entity = Person
    label = "Illinois HBIS income as share of federal poverty level"
    unit = "/1"
    definition_period = YEAR
    defined_for = StateCode.IL
    reference = ("https://www.dhs.state.il.us/page.aspx?item=161600",)
    # HBIS uses AABD methodology for income counting but compares against
    # 100% of the federal poverty level.

    def formula(person, period, parameters):
        # Get monthly AABD-methodology income and annualize
        income = person("il_hbi_senior_countable_income", period)

        # Get household FPL based on SPM unit size
        fpg = person.spm_unit("spm_unit_fpg", period)

        # Return income as fraction of FPL (handle divide by zero)
        return where(fpg > 0, income / fpg, 0)



from policyengine_us.model_api import *


class is_slmb_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Specified Low-Income Medicare Beneficiary (SLMB) eligible"
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/cfr/text/42/435.122",
        "https://www.medicare.gov/basics/costs/help/medicare-savings-programs",
    )

    def formula(person, period, parameters):
        # SLMB requires income above 100% FPL but at or below 120% FPL
        p = parameters(
            period
        ).gov.hhs.medicare.savings_programs.eligibility.income

        medicare_eligible = person("is_medicare_eligible", period.this_year)
        asset_eligible = person("msp_asset_eligible", period)

        fpg = person("msp_fpg", period)
        countable_income = person("msp_countable_income", period)

        qmb_income_limit = fpg * p.qmb.fpl_limit
        slmb_income_limit = fpg * p.slmb.fpl_limit

        income_above_qmb = countable_income > qmb_income_limit
        income_at_or_below_slmb = countable_income <= slmb_income_limit
        income_eligible = income_above_qmb & income_at_or_below_slmb

        return medicare_eligible & income_eligible & asset_eligible



from policyengine_us.model_api import *


class mn_military_pension_subtraction(Variable):
    value_type = float
    entity = TaxUnit
    label = "Minnesota Military Pension Subtraction"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.revisor.mn.gov/statutes/cite/290.0132#stat.290.0132.21",  # Subd. 21 - Military service pension; retirement pay
        "https://www.revenue.state.mn.us/sites/default/files/2025-12/m1m-25.pdf",
    )
    defined_for = StateCode.MN
    adds = ["military_retirement_pay", "military_retirement_pay_survivors"]



from policyengine_us.model_api import *


class ga_tanf_childcare_deduction(Variable):
    value_type = float
    entity = SPMUnit
    label = "Georgia TANF childcare deduction"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://pamms.dhs.ga.gov/dfcs/tanf/1615/",
        "https://pamms.dhs.ga.gov/dfcs/tanf/1605/",
    )
    defined_for = StateCode.GA

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ga.dfcs.tanf.income.deductions
        person = spm_unit.members
        dependent = person("is_tax_unit_dependent", period)
        age = person("monthly_age", period)
        childcare_expenses = spm_unit("childcare_expenses", period)

        # PAMMS 1615: "$200 monthly for each child under the age of two"
        # and "$175 monthly for each individual age two or above"
        # PAMMS 1605: Childcare is an earned income deduction applied
        # in order after work expense
        # Calculate max deduction per dependent based on age
        childcare_deduction_person = p.childcare.calc(age) * dependent
        total_childcare_deduction = spm_unit.sum(childcare_deduction_person)

        return min_(childcare_expenses, total_childcare_deduction)



from policyengine_us.model_api import *


class capped_heat_pump_rebate(Variable):
    value_type = float
    entity = TaxUnit
    label = "Capped heat pump rebate"
    documentation = "Before total high efficiency electric home rebate cap"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        percent_covered = tax_unit(
            "high_efficiency_electric_home_rebate_percent_covered", period
        )
        expenditures = tax_unit("heat_pump_expenditures", period)
        cap = parameters(
            period
        ).gov.doe.high_efficiency_electric_home_rebate.cap.heat_pump
        return min_(expenditures * percent_covered, cap)



from policyengine_us.model_api import *


class ct_income_tax_low_tax_recapture(Variable):
    value_type = float
    entity = TaxUnit
    label = "Connecticut income tax recapture at low brackets"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.CT

    def formula(tax_unit, period, parameters):
        income = tax_unit("ct_agi", period)
        p = parameters(period).gov.states.ct.tax.income.recapture.low
        if p.in_effect:
            filing_status = tax_unit("filing_status", period)
            reduction_start = p.start[filing_status]
            max_amount = p.max_amount[filing_status]
            increment = p.increment[filing_status]
            reduction_per_increment = p.amount[filing_status]
            excess = max_(income - reduction_start, 0)
            increments = np.ceil(excess / increment)
            amount = increments * reduction_per_increment
            return min_(max_amount, amount)
        return 0



from policyengine_us.model_api import *


class nm_works_maximum_benefit(Variable):
    value_type = float
    entity = SPMUnit
    label = "New Mexico Works maximum benefit (payment standard)"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.srca.nm.gov/parts/title08/08.102.0500.html",
        "https://www.hca.nm.gov/2023/09/01/state-announces-a-23-percent-increase-in-cash-assistance-for-low-income-new-mexico-families/",
    )
    defined_for = StateCode.NM

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.nm.hca.nm_works.payment_standard
        unit_size = spm_unit("spm_unit_size", period.this_year)
        capped_size = min_(unit_size, p.max_table_size)
        base_amount = p.amount[capped_size]
        extra_persons = max_(unit_size - p.max_table_size, 0)
        return base_amount + extra_persons * p.additional_person



from policyengine_us.model_api import *


class WIWorksPlacement(Enum):
    CSJ = "Community Service Job"
    W2_TRANSITION = "W-2 Transition"
    CMC = "Custodial Parent of Infant"
    ARP = "At Risk Pregnancy"
    NONE = "None"


class wi_works_placement(Variable):
    value_type = Enum
    entity = SPMUnit
    label = "Wisconsin Works placement type"
    definition_period = MONTH
    possible_values = WIWorksPlacement
    default_value = WIWorksPlacement.CSJ
    defined_for = StateCode.WI
    reference = (
        "https://docs.legis.wisconsin.gov/statutes/statutes/49/iii/148",
        "https://dcf.wisconsin.gov/manuals/w-2-manual/Production/07/7.4.1_Community_Service_Jobs_(CSJ).htm",
    )

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.wi.dcf.works.placement
        person = spm_unit.members

        # CMC: Custodial Parent of Infant (child <= 8 weeks ≈ 2 months)
        child_age_months = person("monthly_age", period)
        has_infant = spm_unit.any(child_age_months <= p.cmc_infant_age_limit)

        # ARP: At Risk Pregnancy (3rd trimester = month 7+)
        is_pregnant = person("is_pregnant", period)
        pregnancy_month = person("current_pregnancy_month", period)
        in_third_trimester = pregnancy_month >= p.arp_pregnancy_month_threshold
        is_arp_eligible = is_pregnant & in_third_trimester
        has_third_trimester = spm_unit.any(is_arp_eligible)

        # Priority: CMC > ARP > CSJ (W2_T requires agency assessment)
        return select(
            [has_infant, has_third_trimester],
            [WIWorksPlacement.CMC, WIWorksPlacement.ARP],
            default=WIWorksPlacement.CSJ,
        )



from policyengine_us.model_api import *


class la_fitap_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Louisiana FITAP eligible"
    definition_period = MONTH
    reference = (
        "https://www.doa.la.gov/media/tp3lmkyg/67.pdf#page=37",
        "https://www.doa.la.gov/media/tp3lmkyg/67.pdf#page=38",
    )
    defined_for = StateCode.LA

    def formula(spm_unit, period, parameters):
        # Per LAC 67:III.1221: Child must be under 18 or 18 and in school
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)

        # Per LAC 67:III.1223: Must be US citizen or qualified alien
        immigration_eligible = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )

        # Per LAC 67:III.1229: Countable income <= flat grant
        income_eligible = spm_unit("la_fitap_income_eligible", period)

        return demographic_eligible & immigration_eligible & income_eligible



from policyengine_us.model_api import *


class is_non_english_speaking_home(Variable):
    value_type = bool
    entity = Household
    label = "Non-English speaking home"
    definition_period = YEAR
    documentation = "Whether the household's primary language is not English"



from policyengine_us.model_api import *


class il_pi_basic_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Meets basic eligibility for Illinois PI (age and income)"
    definition_period = YEAR
    reference = "https://www.isbe.net/Pages/Birth-to-Age-3-Years.aspx"
    defined_for = StateCode.IL

    def formula(person, period, parameters):
        # Basic eligibility: demographic eligible (child under 3 or pregnant)
        # and income eligible (≤400% FPL).
        demographic_eligible = person("il_pi_demographic_eligible", period)
        income_eligible = person("il_isbe_income_eligible", period)
        return demographic_eligible & income_eligible



from policyengine_us.model_api import *


class ri_works_payment_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Rhode Island Works payment standard"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://rules.sos.ri.gov/Regulations/part/218-20-00-2",
        "https://dhs.ri.gov/press-releases/rhode-island-works-families-see-increase-benefits-fy-2025-state-budget",
    )
    defined_for = StateCode.RI

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ri.dhs.works
        unit_size = spm_unit("spm_unit_size", period)
        max_table_size = p.payment_standard.max_unit_size
        capped_size = min_(unit_size, max_table_size)
        base_amount = p.payment_standard.amount[capped_size]

        # Add additional amount per person beyond max table size
        additional_persons = max_(unit_size - max_table_size, 0)
        additional_amount = (
            additional_persons * p.payment_standard.additional_person
        )

        standard = base_amount + additional_amount

        # Apply subsidized housing reduction
        receives_housing = spm_unit(
            "receives_housing_assistance", period.this_year
        )
        housing_reduction = where(
            receives_housing, p.subsidized_housing_reduction, 0
        )

        return max_(standard - housing_reduction, 0)



from policyengine_us.model_api import *


class parent_has_less_than_high_school_education(Variable):
    value_type = bool
    entity = Household
    label = "Parent has less than high school education"
    definition_period = YEAR
    documentation = "Whether any parent or guardian in the household has less than a high school diploma or equivalent"



from policyengine_us.model_api import *


class ar_tea_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Arkansas TEA countable earned income"
    unit = USD
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/arkansas/208-00-13-Ark-Code-R-SS-001"
    defined_for = StateCode.AR

    def formula(spm_unit, period, parameters):
        # Per 208.00.13 Ark. Code R. Section 001, Section 3.3
        p = parameters(period).gov.states.ar.dhs.tea.income.work_deduction
        gross_earned = add(spm_unit, period, ["tanf_gross_earned_income"])

        # Step 1: Apply 20% work expense deduction (applies to all)
        after_work_expense = gross_earned * (1 - p.expense_rate)

        # Step 2: Apply 60% work incentive deduction (only for ongoing recipients)
        is_enrolled = spm_unit("is_tanf_enrolled", period)

        # For initial applicants: only 20% work expense deduction
        # For ongoing recipients: 20% work expense + 60% work incentive
        return where(
            is_enrolled,
            after_work_expense * (1 - p.incentive_rate),
            after_work_expense,
        )



from policyengine_us.model_api import *


class il_ipass_assist_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    definition_period = YEAR
    label = "Illinois I-PASS Assist eligible"
    reference = (
        "https://agency.illinoistollway.com/assist",
        "https://www.dhs.state.il.us/page.aspx?item=150431",
    )
    defined_for = StateCode.IL

    def formula(spm_unit, period, parameters):
        income_eligible = spm_unit("il_ipass_assist_income_eligible", period)
        categorical_eligible = spm_unit(
            "il_ipass_assist_categorical_eligible", period
        )
        return income_eligible | categorical_eligible



from policyengine_us.model_api import *


class is_emergency_medicaid_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Eligible for Emergency Medicaid"
    definition_period = YEAR
    reference = (
        "https://www.law.cornell.edu/uscode/text/42/1396b#v_2",
        "https://www.law.cornell.edu/cfr/text/42/435.406",
    )

    def formula(person, period, parameters):
        p = parameters(period).gov.hhs.medicaid.emergency_medicaid

        has_emergency_condition = person(
            "has_emergency_medical_condition", period
        )
        medicaid_category = person("medicaid_category", period)
        meets_medicaid_requirements = (
            medicaid_category != medicaid_category.possible_values.NONE
        )
        immigration_status_eligible = person(
            "is_medicaid_immigration_status_eligible", period
        )

        return (
            p.enabled
            & has_emergency_condition
            & meets_medicaid_requirements
            & ~immigration_status_eligible
        )



from policyengine_us.model_api import *


class ak_atap(Variable):
    value_type = float
    entity = SPMUnit
    label = "Alaska ATAP benefit"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/alaska/7-AAC-45.525",
        "https://www.law.cornell.edu/regulations/alaska/7-AAC-45.523",
    )
    defined_for = "ak_atap_eligible"

    def formula(spm_unit, period, parameters):
        # NOTE: The following Alaska ATAP features are not modeled:
        #
        # Unit types not modeled:
        #   1. Child-only units - Only adult-included units are implemented
        #   2. Two-parent incapacitated - Both parents treated as able to work
        #
        # Benefit adjustments not modeled:
        #   3. Shelter cost reduction - Benefits reduced for families with low
        #      shelter costs based on regional utility standards
        #   4. Summer reduction - Two-parent families receive reduced benefits
        #      during summer months (July-September)
        #
        # See 7 AAC 45.520-525 for full eligibility and benefit rules.

        p = parameters(period).gov.states.ak.dpa.atap
        need_standard = spm_unit("ak_atap_need_standard", period)
        countable_income = spm_unit("ak_atap_countable_income", period)

        # Per 7 AAC 45.525: Payment = (Need Standard - Countable Income)
        # multiplied by (size 2 max payment / size 2 need standard)
        # The ratio uses fixed size-2 values, not the family's actual size
        income_deficit = max_(need_standard - countable_income, 0)
        size_2_max = p.payment.base
        # Per 7 AAC 45.525: ratable reduction formula always uses size-2 as base
        size_2_need = p.need_standard.amount["2"]
        ratable_reduction = size_2_max / size_2_need
        return income_deficit * ratable_reduction



from policyengine_us.model_api import *


class ne_adc_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Nebraska ADC countable earned income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://nebraskalegislature.gov/laws/statutes.php?statute=68-1726"
    )
    defined_for = StateCode.NE

    def formula(spm_unit, period, parameters):
        p = parameters(
            period
        ).gov.states.ne.dhhs.adc.income.earned_income_disregard
        gross_earned = add(spm_unit, period, ["tanf_gross_earned_income"])
        is_enrolled = spm_unit("is_tanf_enrolled", period)
        # Per Neb. Rev. Stat. 68-1726(3)(a):
        # (i) 20% disregard for initial applicants
        # (ii) 50% disregard for ongoing recipients
        disregard_rate = where(is_enrolled, p.ongoing, p.initial)
        return gross_earned * (1 - disregard_rate)



from policyengine_us.model_api import *


class ut_fep_net_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Utah TANF under net income test"
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/utah/Utah-Admin-Code-R986-200-239"
    defined_for = StateCode.UT

    def formula(spm_unit, period, parameters):
        # Utah net income test: countable income < 100% of SNB (strict less than)
        p = parameters(period).gov.states.ut.dwf.fep

        size = spm_unit("spm_unit_size", period)
        size_capped = min_(size, p.payment_standard.max_unit_size)

        countable_income = spm_unit("ut_fep_countable_income", period)
        return countable_income < p.standard_needs_budget.amount[size_capped]



from policyengine_us.model_api import *


class mi_fip_earned_income_after_deductions_for_eligibility_person(Variable):
    value_type = float
    entity = Person
    label = "Michigan FIP earned income after deductions for eligibility per person"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.MI
    reference = (
        "https://mdhhs-pres-prod.michigan.gov/olmweb/ex/BP/Public/BEM/518.pdf",
        "https://mdhhs-pres-prod.michigan.gov/olmweb/EX/BP/Public/BEM/520.pdf",
    )

    def formula(person, period, parameters):
        p = parameters(period).gov.states.mi.mdhhs.fip.income.disregard

        # Get gross earned income for this person
        gross_earned = person("tanf_gross_earned_income", period)

        # BEM 518 Page 5 - "Qualifying Earned Income Disregard"
        # BEM 520 Section C - Qualifying Income Test (for initial eligibility)
        # "At application, deduct $200 from each person's countable earnings.
        # Then deduct an additional 20 percent of each person's remaining earnings."

        # Step 1: Deduct $200 from this person's earnings
        flat_deduction = p.flat
        remainder = max_(gross_earned - flat_deduction, 0)

        # Step 2: Deduct 20% of remainder (for initial eligibility test)
        percent_deduction = remainder * p.initial_rate

        # Step 3: Total deduction
        total_deduction = flat_deduction + percent_deduction

        # Step 4: Countable income (cannot be negative)
        return max_(gross_earned - total_deduction, 0)



from policyengine_us.model_api import *


class wi_works_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Wisconsin Works countable income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://dcf.wisconsin.gov/manuals/w-2-manual/Production/03/03.2.7_Counting_Income.htm",
        "https://dcf.wisconsin.gov/manuals/w-2-manual/Production/03/03.2.8_DISREGARDING_INCOME.htm",
    )
    defined_for = StateCode.WI
    # Wisconsin disregards child support, which is included in
    # tanf_gross_unearned_income. The result cannot be negative.
    adds = ["tanf_gross_earned_income", "tanf_gross_unearned_income"]
    subtracts = ["child_support_received"]



from policyengine_us.model_api import *


class ks_tanf_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Kansas TANF countable income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/kansas/K-A-R-30-4-110",
        "https://content.dcf.ks.gov/ees/keesm/current/keesm7110.htm",
    )
    defined_for = StateCode.KS
    adds = [
        "ks_tanf_countable_earned_income",
        "tanf_gross_unearned_income",
    ]



from policyengine_us.model_api import *


class ar_tea_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Arkansas TEA countable income"
    unit = USD
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/arkansas/208-00-13-Ark-Code-R-SS-001"
    defined_for = StateCode.AR

    adds = ["ar_tea_countable_earned_income", "tanf_gross_unearned_income"]



from policyengine_us.model_api import *


class ne_adc_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Nebraska ADC eligible"
    definition_period = MONTH
    reference = (
        "https://dhhs.ne.gov/Pages/TANF.aspx",
        "https://dhhs.ne.gov/Pages/Title-468.aspx",
    )
    defined_for = StateCode.NE

    def formula(spm_unit, period, parameters):
        # Use federal demographic eligibility baseline
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        # Check immigration eligibility (at least one citizen/legal immigrant)
        immigration_eligible = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )
        # Income eligibility
        income_eligible = spm_unit("ne_adc_income_eligible", period)
        # Resource eligibility (YEAR variable)
        resources_eligible = spm_unit(
            "ne_adc_resources_eligible", period.this_year
        )
        return (
            demographic_eligible
            & immigration_eligible
            & income_eligible
            & resources_eligible
        )



from policyengine_us.model_api import *


class wv_works_resources_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "West Virginia WV Works resources eligible"
    definition_period = MONTH
    reference = "https://dhhr.wv.gov/bcf/Services/familyassistance/Documents/726/726%20ch11_1.pdf"
    defined_for = StateCode.WV

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.wv.dhhr.works.resources
        # spm_unit_assets is a YEAR variable
        countable_resources = spm_unit("spm_unit_assets", period.this_year)
        return countable_resources <= p.limit



from policyengine_us.model_api import *


class il_pi_priority_score(Variable):
    value_type = int
    entity = Person
    label = "Weighted priority score for Illinois PI eligibility"
    definition_period = YEAR
    reference = (
        "https://www.isbe.net/Documents/Prevention-Initiative-Eligibility-Form.pdf#page=2",
        "https://www.isbe.net/Documents/Prevention-Initiative-Eligibility-Form.pdf#page=3",
    )
    defined_for = "il_pi_demographic_eligible"

    adds = [
        "il_pi_highest_priority_score",
        "il_pi_other_priority_score",
        "il_pi_lower_priority_score",
    ]



from policyengine_us.model_api import *


class ne_adc_maximum_benefit(Variable):
    value_type = float
    entity = SPMUnit
    label = "Nebraska ADC maximum benefit before unearned income deduction"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://nebraskalegislature.gov/laws/statutes.php?statute=68-1713",
        "https://dhhs.ne.gov/Pages/Title-468.aspx",
    )
    defined_for = StateCode.NE

    def formula(spm_unit, period, parameters):
        need_standard = spm_unit("ne_adc_need_standard", period)
        countable_earned = spm_unit("ne_adc_countable_earned_income", period)
        payment_standard = spm_unit("ne_adc_payment_standard", period)
        # Gap budgeting: Gap = Standard of Need - Net Earned Income
        gap = max_(need_standard - countable_earned, 0)
        # Benefit = min(Gap, Payment Standard)
        return min_(gap, payment_standard)



from policyengine_us.model_api import *


class nh_fanf_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "New Hampshire FANF eligible"
    definition_period = MONTH
    reference = (
        "https://gc.nh.gov/rsa/html/xii/167/167-79.htm",
        "https://www.dhhs.nh.gov/sr_htm/html/sr_97-03_dated_02_97.htm",
    )
    defined_for = StateCode.NH

    def formula(spm_unit, period, parameters):
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        immigration_eligible = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )
        income_eligible = spm_unit("nh_fanf_income_eligible", period)
        resources_eligible = spm_unit(
            "nh_fanf_resources_eligible", period.this_year
        )
        return (
            demographic_eligible
            & immigration_eligible
            & income_eligible
            & resources_eligible
        )



from policyengine_us.model_api import *


class wy_power_earned_income_disregard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Wyoming POWER earned income disregard"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://wyoleg.gov/statutes/compress/title42.pdf#page=10",
        "https://dfs.wyo.gov/about/policy-manuals/snap-and-power-policy-manual/",
    )
    defined_for = StateCode.WY

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.wy.dfs.power.income.disregard
        # Per W.S. 42-2-103(a)(iv): $600 for any one recipient,
        # $1,200 for married couples
        is_married = add(spm_unit, period, ["is_tax_unit_head_or_spouse"]) > 1
        return where(is_married, p.married_couple, p.individual)



from policyengine_us.model_api import *


class fl_tca_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Florida TCA countable income"
    unit = USD
    definition_period = MONTH
    reference = "https://www.leg.state.fl.us/statutes/index.cfm?App_mode=Display_Statute&URL=0400-0499/0414/Sections/0414.095.html"
    defined_for = StateCode.FL
    adds = [
        "fl_tca_countable_earned_income",
        "tanf_gross_unearned_income",
    ]



from policyengine_us.model_api import *


class pa_tanf_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Pennsylvania TANF countable earned income"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.PA
    reference = (
        "http://services.dpw.state.pa.us/oimpolicymanuals/cash/160_Income_Deductions/160_2_TANF_Earned_Income_Deductions.htm",
        "https://www.law.cornell.edu/regulations/pennsylvania/55-Pa-Code-SS-183-94",
    )
    adds = ["pa_tanf_earned_income_after_deductions_person"]



from policyengine_us.model_api import *


class msp_asset_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Medicare Savings Program asset eligible"
    definition_period = MONTH
    reference = (
        "https://www.medicare.gov/basics/costs/help/medicare-savings-programs",
        "https://www.medicareinteractive.org/understanding-medicare/"
        "cost-saving-programs/medicare-savings-programs-qmb-slmb-qi/"
        "medicare-savings-program-income-and-asset-limits",
    )

    def formula(person, period, parameters):
        p = parameters(period).gov.hhs.medicare.savings_programs.eligibility
        state_code = person.household("state_code_str", period)
        # Check if asset test applies (some states have eliminated it)
        asset_test_applies = p.asset.applies[state_code]
        # If asset test doesn't apply, everyone is asset-eligible
        cash_assets = person.spm_unit("spm_unit_cash_assets", period.this_year)
        married = person.spm_unit("spm_unit_is_married", period)
        asset_limit = where(married, p.asset.couple, p.asset.individual)
        meets_asset_test = cash_assets <= asset_limit
        return where(asset_test_applies, meets_asset_test, True)



from policyengine_us.model_api import *


class was_calworks_recipient(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Was a CalWORKs cash aid recipient"
    definition_period = YEAR
    defined_for = StateCode.CA
    default_value = False
    reference = "https://leginfo.legislature.ca.gov/faces/codes_displaySection.xhtml?sectionNum=10372.&nodeTreePath=16.4.19&lawCode=WIC"



from policyengine_us.model_api import *


class mo_tanf_resources_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Missouri TANF resource eligibility"
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/missouri/13-CSR-40-2-310",
        "https://dssmanuals.mo.gov/temporary-assistance-case-management/0200-000-00/",
    )
    defined_for = StateCode.MO

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.mo.dss.tanf.resource_limit
        resources = spm_unit("spm_unit_cash_assets", period.this_year)
        return resources <= p.amount



from policyengine_us.model_api import *


class ut_fep_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Utah Family Employment Program"
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/utah/Utah-Admin-Code-R986-200-202"
    defined_for = StateCode.UT

    def formula(spm_unit, period, parameters):
        # Per R986-200: Must meet demographic, income, and resource tests
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        income_eligible = spm_unit("ut_fep_income_eligible", period)
        resource_eligible = spm_unit("ut_fep_resources_eligible", period)
        return demographic_eligible & income_eligible & resource_eligible



from policyengine_us.model_api import *


class mn_k12_qualifying_children(Variable):
    value_type = int
    entity = TaxUnit
    label = "Minnesota K-12 qualifying children count"
    definition_period = YEAR
    reference = (
        "https://www.revisor.mn.gov/statutes/cite/290.0674",
        "https://www.revenue.state.mn.us/sites/default/files/2025-12/m1ed-25.pdf",
    )
    defined_for = StateCode.MN

    def formula(tax_unit, period, parameters):
        # A qualifying child must:
        # - Be in grades K-12
        # - Be a dependent
        # - Have lived with the filer more than half the year
        # Using is_in_k12_school and is_tax_unit_dependent as proxies
        is_k12 = tax_unit.members("is_in_k12_school", period)
        is_dependent = tax_unit.members("is_tax_unit_dependent", period)
        return tax_unit.sum(is_k12 & is_dependent)



from policyengine_us.model_api import *


class mn_k12_education_credit_eligible(Variable):
    value_type = bool
    entity = TaxUnit
    label = "Eligible for Minnesota K-12 Education Credit"
    definition_period = YEAR
    reference = (
        "https://www.revisor.mn.gov/statutes/cite/290.0674",
        "https://www.revenue.state.mn.us/sites/default/files/2025-12/m1ed-25.pdf",
    )
    defined_for = StateCode.MN

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.states.mn.tax.income.credits.k12_education
        # Check eligibility - not available for married filing separately
        filing_status = tax_unit("filing_status", period)
        is_separate = filing_status == filing_status.possible_values.SEPARATE
        # Count qualifying K-12 children
        k12_children = tax_unit("mn_k12_qualifying_children", period)
        # Check income limit based on number of children
        agi = tax_unit("adjusted_gross_income", period)
        # Income limit = base + additional * max(0, children - child_threshold)
        income_limit = p.income_limit.base + p.income_limit.additional * max_(
            0, k12_children - p.income_limit.child_threshold
        )
        income_eligible = agi < income_limit
        # Eligible if: not married filing separately, has qualifying children,
        # and income is below the limit
        return ~is_separate & income_eligible & (k12_children > 0)



from policyengine_us.model_api import *


class fl_tca_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Florida TCA countable earned income"
    unit = USD
    definition_period = MONTH
    reference = "https://www.leg.state.fl.us/statutes/index.cfm?App_mode=Display_Statute&URL=0400-0499/0414/Sections/0414.095.html"
    defined_for = StateCode.FL

    def formula(spm_unit, period, parameters):
        # Per DCF 2620.0111.01: Income Test and Benefit Determination
        # Step 2: Subtract $90 standard disregard
        # Step 3: Subtract $200 and 1/2 of remainder
        p = parameters(period).gov.states.fl.dcf.tanf.income.disregard
        gross_earned = add(spm_unit, period, ["tanf_gross_earned_income"])

        # Step 2: Subtract $90 standard disregard
        after_standard = max_(gross_earned - p.standard_disregard, 0)

        # Step 3: Subtract $200, then 50% of remainder is countable
        remaining_after_first = max_(after_standard - p.first_amount, 0)
        return remaining_after_first * (1 - p.remaining_rate)



from policyengine_us.model_api import *


class il_isbe_was_teen_parent_at_first_birth(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Parent was a teen at birth of first child"
    definition_period = YEAR
    reference = (
        "https://www.isbe.net/Documents/pdg-eg-grant-enrollment-form.pdf"
    )
    defined_for = StateCode.IL

    def formula(spm_unit, period, parameters):
        person = spm_unit.members
        age = person("age", period.this_year)
        is_child = person("is_child", period.this_year)

        # Find oldest child using get_rank (rank 0 = oldest when using -age)
        is_oldest_child = person.get_rank(spm_unit, -age, is_child) == 0

        # Get ages
        head_age = spm_unit.max(age * person("is_tax_unit_head", period))
        oldest_child_age = spm_unit.max(age * is_oldest_child)

        # Calculate parent's age at first birth
        age_at_first_birth = head_age - oldest_child_age

        # Check if parent was under threshold age at birth and has children
        p = parameters(period).gov.states.il.isbe.age_threshold
        has_children = spm_unit.any(is_child)
        return has_children & (age_at_first_birth < p.teen_parent)



from policyengine_us.model_api import *


class or_tanf_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Oregon TANF countable income"
    unit = USD
    definition_period = MONTH
    reference = "https://oregon.public.law/rules/oar_461-140-0010"
    defined_for = StateCode.OR

    def formula(spm_unit, period, parameters):
        gross_earned = add(spm_unit, period, ["tanf_gross_earned_income"])
        gross_unearned = add(spm_unit, period, ["tanf_gross_unearned_income"])
        child_support_disregard = spm_unit(
            "or_tanf_child_support_disregard", period
        )
        countable_unearned = max_(gross_unearned - child_support_disregard, 0)
        return gross_earned + countable_unearned



from policyengine_us.model_api import *


class ky_ktap_resource_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Kentucky K-TAP due to resources"
    definition_period = MONTH
    reference = "https://apps.legislature.ky.gov/law/kar/titles/921/002/016/"
    defined_for = StateCode.KY

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ky.dcbs.ktap.eligibility
        resources = spm_unit("spm_unit_assets", period.this_year)
        return resources <= p.resource_limit



from policyengine_us.model_api import *


class ca_riv_general_relief_ineligible_person(Variable):
    value_type = bool
    entity = Person
    label = "Ineligible person for the Riverside County General Relief"
    definition_period = MONTH
    defined_for = "in_riv"

    def formula(person, period, parameters):
        # SSI is person-level: only the individual receiving SSI is excluded
        receives_ssi = person("ssi", period) > 0
        # TANF (CalWORKs) is unit-level: if unit receives TANF, members are excluded
        receives_tanf = person.spm_unit("ca_tanf", period) > 0
        receives_ssi_or_tanf = receives_ssi | receives_tanf
        immigration_status_eligible = person(
            "ca_riv_general_relief_immigration_status_eligible", period
        )
        meets_work_requirement = person(
            "ca_riv_general_relief_meets_work_requirements", period
        )
        return (
            receives_ssi_or_tanf
            | ~immigration_status_eligible
            | ~meets_work_requirement
        )



from policyengine_us.model_api import *


class il_pi_risk_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Meets risk factor requirements for Illinois PI"
    definition_period = YEAR
    reference = (
        "https://www.isbe.net/Documents/Prevention-Initiative-Eligibility-Form.pdf#page=3",
    )
    defined_for = StateCode.IL

    def formula(person, period, parameters):
        # Eligible if priority score meets minimum (50 points).
        p = parameters(period).gov.states.il.isbe.pi.eligibility.risk_factors
        priority_score = person("il_pi_priority_score", period)
        return priority_score >= p.minimum_score



from policyengine_us.model_api import *


class ct_tfa_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Connecticut Temporary Family Assistance (TFA) due to income"
    definition_period = MONTH
    reference = "https://portal.ct.gov/dss/-/media/departments-and-agencies/dss/state-plans-and-federal-reports/tanf-state-plan/ct-tanf-state-plan-2024---2026---41524-amendment.pdf#page=10"
    defined_for = StateCode.CT

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ct.dss.tfa.income.recipient
        is_tanf_enrolled = spm_unit("is_tanf_enrolled", period)
        # New applicant income check
        countable_income_at_application = spm_unit(
            "ct_tfa_countable_income_at_application", period
        )
        need_standard = spm_unit("ct_tfa_need_standard", period)
        income_eligible_at_application = (
            countable_income_at_application <= need_standard
        )
        # Existing recipient income check
        # Pre 2024, gross earning <= 100% fpg
        # Starting 2024, gross earning <= 230% FPL
        gross_earnings = add(spm_unit, period, ["tanf_gross_earned_income"])
        fpg = spm_unit("tanf_fpg", period)

        fpg_limit = p.income_limit_rate * fpg
        income_eligible_at_grant_calculation = gross_earnings <= fpg_limit
        return where(
            is_tanf_enrolled,
            income_eligible_at_grant_calculation,
            income_eligible_at_application,
        )



from policyengine_us.model_api import *


class wi_works_resources_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Wisconsin Works resources eligible"
    definition_period = YEAR
    reference = (
        "https://dcf.wisconsin.gov/manuals/w-2-manual/Production/03/03.3.4_COUNTING_ASSETS.htm",
        "https://docs.legis.wisconsin.gov/code/admin_code/dcf/101_199/101/09/3/b",
    )
    defined_for = StateCode.WI

    def formula(spm_unit, period, parameters):
        countable = spm_unit("wi_works_countable_resources", period)
        p = parameters(period).gov.states.wi.dcf.works.asset
        return countable <= p.limit



from policyengine_us.model_api import *


class ca_child_care_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "California child care countable income"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.CA
    reference = (
        "https://www.law.cornell.edu/regulations/california/5-CCR-18078"
    )

    adds = "gov.states.ca.cdss.child_care.income.sources"



from policyengine_us.model_api import *


class or_tanf_payment_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Oregon TANF payment standard"
    unit = USD
    definition_period = MONTH
    reference = "https://oregon.public.law/rules/oar_461-155-0030"
    defined_for = StateCode.OR

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states["or"].odhs.tanf
        size = spm_unit("spm_unit_size", period.this_year)
        max_size = p.maximum_need_group_size
        size_capped = min_(size, max_size)
        additional_people = max_(size - max_size, 0)
        base_amount = p.payment_standard.amount[size_capped]
        additional_amount = (
            additional_people * p.payment_standard.additional_person
        )
        return base_amount + additional_amount



from policyengine_us.model_api import *


class de_tanf_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Delaware TANF"
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/delaware/16-Del-Admin-Code-SS-4000-4008",
        "https://dhss.delaware.gov/dss/tanf/",
    )
    defined_for = StateCode.DE

    def formula(spm_unit, period, parameters):
        # Use federal demographic eligibility (minor child or pregnant)
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        # Delaware follows federal immigration rules
        immigration_eligible = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )
        # State-specific eligibility tests
        gross_income_eligible = spm_unit(
            "de_tanf_gross_income_eligible", period
        )
        net_income_eligible = spm_unit("de_tanf_net_income_eligible", period)
        resources_eligible = spm_unit("de_tanf_resources_eligible", period)

        return (
            demographic_eligible
            & immigration_eligible
            & gross_income_eligible
            & net_income_eligible
            & resources_eligible
        )



from policyengine_us.model_api import *


class sd_tanf_resources_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "South Dakota TANF resources eligible"
    definition_period = MONTH
    reference = "https://sdlegislature.gov/Rules/Administrative/67:10:04:03"
    defined_for = StateCode.SD

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.sd.dss.tanf.resources
        resources = spm_unit("spm_unit_assets", period.this_year)
        return resources <= p.limit



from policyengine_us.model_api import *


class de_tanf_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Delaware TANF countable income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/delaware/16-Del-Admin-Code-SS-4000-4008",
        "https://dhss.delaware.gov/wp-content/uploads/sites/11/dss/pdf/detanfstateplan2017.pdf#page=6",
    )
    defined_for = StateCode.DE

    # For recipient eligibility and benefit calculation:
    # Countable earned ($90 + $30 + 1/3 + childcare) + countable unearned
    adds = [
        "de_tanf_countable_earned_income",
        "de_tanf_countable_unearned_income",
    ]



from policyengine_us.model_api import *


class in_tanf_countable_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Indiana TANF countable income eligible"
    definition_period = MONTH
    reference = (
        "https://iar.iga.in.gov/code/2026/470/10.3#470-10.3-4-1",
        "https://iga.in.gov/laws/2025/ic/titles/12/#12-14-1-1.7",
    )
    defined_for = StateCode.IN

    def formula(spm_unit, period, parameters):
        # Initial: standard of need; Continuing: FPL * rate
        p = parameters(period).gov.states["in"].fssa.tanf
        countable = spm_unit(
            "in_tanf_countable_income_for_eligibility", period
        )
        is_enrolled = spm_unit("is_tanf_enrolled", period)

        fpg = spm_unit("tanf_fpg", period)
        continuing_threshold = fpg * p.eligibility.continuing.fpg_rate

        threshold = where(
            is_enrolled,
            continuing_threshold,
            spm_unit("in_tanf_payment_standard", period),
        )
        return countable < threshold



from policyengine_us.model_api import *


class ak_atap_gross_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Alaska ATAP gross income eligible"
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/alaska/7-AAC-45.470"
    defined_for = StateCode.AK

    def formula(spm_unit, period, parameters):
        # Test 1: Total nonexempt gross monthly income <= 185% standard
        gross_income = spm_unit("ak_atap_gross_income", period)
        gross_income_limit = spm_unit("ak_atap_gross_income_limit", period)
        return gross_income <= gross_income_limit



from policyengine_us.model_api import *


class me_tanf_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Maine TANF"
    definition_period = MONTH
    reference = (
        "https://www.mainelegislature.org/legis/statutes/22/title22sec3762.html",
        "https://www.maine.gov/dhhs/ofi/programs-services/tanf",
    )
    defined_for = StateCode.ME

    def formula(spm_unit, period, parameters):
        # Per 22 M.R.S. Section 3762 and 10-144 C.M.R. Chapter 331
        income_eligible = spm_unit("me_tanf_income_eligible", period)
        resources_eligible = spm_unit("me_tanf_resources_eligible", period)
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        immigration_eligible = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )

        return (
            income_eligible
            & resources_eligible
            & demographic_eligible
            & immigration_eligible
        )



from policyengine_us.model_api import *


class ok_tanf_resources_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Oklahoma TANF resources eligible"
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/oklahoma/Okla-Admin-Code-SS-340-10-3-5"
    defined_for = StateCode.OK

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ok.dhs.tanf.resources
        # Per OAC 340:10-3-5: Maximum countable reserve is $1,000
        resources = spm_unit("spm_unit_assets", period.this_year)
        return resources <= p.limit



from policyengine_us.model_api import *


def create_ny_a06774_enhanced_cdcc() -> Reform:
    """
    NY Assembly Bill A06774 - Enhanced Child and Dependent Care Credit

    Increases the NY child and dependent care credit to 110% of the federal
    credit for taxpayers with NY adjusted gross income up to $50,000.

    Reference: https://www.nysenate.gov/legislation/bills/2025/A6774
    """

    class ny_cdcc(Variable):
        value_type = float
        entity = TaxUnit
        label = "NY CDCC"
        unit = USD
        definition_period = YEAR
        reference = (
            "https://www.nysenate.gov/legislation/bills/2025/A6774",
            "https://www.nysenate.gov/legislation/laws/TAX/606",
        )
        defined_for = StateCode.NY

        def formula(tax_unit, period, parameters):
            p = parameters(period).gov.contrib.states.ny.a06774
            ny_agi = tax_unit("ny_agi", period)
            income_threshold = p.income_threshold
            # Calculate the enhanced credit (110% of federal CDCC)
            federal_cdcc = tax_unit("cdcc", period)
            enhanced_cdcc = federal_cdcc * p.match

            # Calculate the standard NY CDCC
            cdcc_max = tax_unit("ny_cdcc_max", period)
            expenses = tax_unit("cdcc_relevant_expenses", period)
            ny_rate = tax_unit("ny_cdcc_rate", period)
            federal_rate = tax_unit("cdcc_rate", period)
            standard_ny_cdcc = min_(
                cdcc_max, expenses * ny_rate * federal_rate
            )

            # Use enhanced credit if reform is in effect and income is
            # at or below the threshold
            eligible_for_enhanced = ny_agi <= income_threshold
            return where(
                eligible_for_enhanced, enhanced_cdcc, standard_ny_cdcc
            )

    class reform(Reform):
        def apply(self):
            self.update_variable(ny_cdcc)

    return reform


def create_ny_a06774_enhanced_cdcc_reform(
    parameters, period, bypass: bool = False
):
    if bypass:
        return create_ny_a06774_enhanced_cdcc()

    p = parameters(period).gov.contrib.states.ny.a06774

    if p.in_effect:
        return create_ny_a06774_enhanced_cdcc()
    else:
        return None


ny_a06774_enhanced_cdcc = create_ny_a06774_enhanced_cdcc_reform(
    None, None, bypass=True
)



from policyengine_us.model_api import *


class or_tanf_child_support_disregard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Oregon TANF child support disregard"
    unit = USD
    definition_period = MONTH
    reference = "https://oregon.public.law/rules/oar_461-145-0080"
    defined_for = StateCode.OR

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states["or"].odhs.tanf
        child_support_received = add(
            spm_unit, period, ["child_support_received"]
        )
        person = spm_unit.members
        age = person("age", period.this_year)
        is_child = age < p.age_threshold.minor_child
        num_children = spm_unit.sum(is_child)
        disregard = p.income.child_support_disregard
        max_disregard_by_children = num_children * disregard.per_child
        max_disregard = min_(max_disregard_by_children, disregard.max)
        return min_(child_support_received, max_disregard)



from policyengine_us.model_api import *


class nm_works_childcare_deduction(Variable):
    value_type = float
    entity = SPMUnit
    label = "New Mexico Works childcare deduction"
    unit = USD
    definition_period = MONTH
    reference = "https://www.srca.nm.gov/parts/title08/08.102.0520.html"
    defined_for = StateCode.NM

    def formula(spm_unit, period, parameters):
        # Per 8.102.520.12(D) NMAC, child care deduction varies by age:
        # Under age 2: up to $200
        # Age 2 or older: up to $175
        p = parameters(
            period
        ).gov.states.nm.hca.nm_works.income.deductions.childcare

        person = spm_unit.members
        is_dependent = person("is_tax_unit_dependent", period)
        age = person("age", period.this_year)
        childcare_expenses = spm_unit("childcare_expenses", period)

        # Max deduction per child based on age
        childcare_max_per_child = p.amount.calc(age) * is_dependent
        total_childcare_max = spm_unit.sum(childcare_max_per_child)

        return min_(childcare_expenses, total_childcare_max)



from policyengine_us.model_api import *


class ut_fep_gross_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Utah TANF under gross income test"
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/utah/Utah-Admin-Code-R986-200-239"
    defined_for = StateCode.UT

    def formula(spm_unit, period, parameters):
        # Utah gross income test: gross income <= 185% of SNB (Standard Needs Budget)
        # Per R986-200-239(1), gross income must be <= 185% of SNB
        # The net income limit equals the SNB
        p = parameters(period).gov.states.ut.dwf.fep

        size = spm_unit("spm_unit_size", period)
        size_capped = min_(size, p.payment_standard.max_unit_size)

        # Standard Needs Budget (SNB) = net income limit
        snb = p.standard_needs_budget.amount[size_capped]
        gross_income_limit = snb * p.income.gross_income_limit.rate

        gross_income = spm_unit("ut_fep_gross_income", period)
        return gross_income <= gross_income_limit



from policyengine_us.model_api import *


class ok_agi_subtractions(Variable):
    value_type = float
    entity = TaxUnit
    label = "Oklahoma AGI subtractions from federal AGI"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://oklahoma.gov/content/dam/ok/en/tax/documents/forms/individuals/past-year/2021/511-Pkt-2021.pdf#page=16"
        "https://oklahoma.gov/content/dam/ok/en/tax/documents/forms/individuals/current/511-Pkt.pdf#page=16"
    )
    defined_for = StateCode.OK

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.states.ok.tax.income.agi.subtractions
        total_subtractions = add(tax_unit, period, p.subtractions)
        # Prevent negative subtractions from acting as additions
        return max_(0, total_subtractions)



from policyengine_us.model_api import *


class has_emergency_medical_condition(Variable):
    value_type = bool
    entity = Person
    label = "Has emergency medical condition"
    definition_period = YEAR
    default_value = False
    reference = (
        "https://www.law.cornell.edu/uscode/text/42/1396b#v_3",
        "https://www.law.cornell.edu/cfr/text/42/440.255#c",
    )
    # Placing the patient's health in serious jeopardy
    # Serious impairment to bodily functions
    # Serious dysfunction of any bodily organ or part



from policyengine_us.model_api import *


class nv_tanf_need_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Nevada TANF need standard"
    unit = USD
    definition_period = MONTH
    reference = "https://dss.nv.gov/uploadedFiles/dwssnvgov/content/Home/Features/eligibility/Chapter%20C_140.pdf"
    defined_for = StateCode.NV

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.nv.dwss.tanf
        size = spm_unit("spm_unit_size", period.this_year)
        max_explicit_size = p.max_unit_size

        # For sizes <= 8, use explicit table values
        # For sizes > 8, use base at size 8 + increment per additional person
        base_amount = p.need_standard.amount[min_(size, max_explicit_size)]
        additional_persons = max_(size - max_explicit_size, 0)
        additional_amount = (
            additional_persons * p.need_standard.additional_person
        )

        return base_amount + additional_amount



from policyengine_us.model_api import *


class mi_fip(Variable):
    value_type = float
    entity = SPMUnit
    label = "Michigan Family Independence Program"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://mdhhs-pres-prod.michigan.gov/olmweb/EX/BP/Public/BEM/520.pdf#page=4",
        "https://mdhhs-pres-prod.michigan.gov/olmweb/ex/BP/Public/BEM/518.pdf#page=5",
    )
    defined_for = "mi_fip_eligible"

    def formula(spm_unit, period, parameters):
        # BEM 520 Section F: Issuance Amount
        # "Subtract line 2 and line 3 from the amount on line 1"
        # Line 1: Payment Standard (from Section B)
        # Line 2: Countable Income (from Section D - uses 50% deduction)
        # Line 3: Recoupment (simplified: excluded)

        payment_standard = spm_unit("mi_fip_payment_standard", period)
        countable_income = spm_unit(
            "mi_fip_countable_income_for_benefit", period
        )

        # BEM 518: Minimum benefit requirement is $10 deficit
        # Negative benefits are set to 0
        return max_(payment_standard - countable_income, 0)



from policyengine_us.model_api import *


class ar_tea_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Arkansas TEA income eligible"
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/arkansas/208-00-13-Ark-Code-R-SS-001"
    defined_for = StateCode.AR

    def formula(spm_unit, period, parameters):
        # Per 208.00.13 Ark. Code R. Section 001, Section 3.3
        p = parameters(period).gov.states.ar.dhs.tea.income
        # Countable income must be at or below the income limit
        countable_income = spm_unit("ar_tea_countable_income", period)
        return countable_income <= p.income_limit



from policyengine_us.model_api import *


class nd_tanf_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "North Dakota TANF countable earned income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.nd.gov/dhs/policymanuals/40019/400_19_110_20.htm",
        "https://www.law.cornell.edu/regulations/north-dakota/N-D-A-C-75-02-1.2-51",
    )
    defined_for = StateCode.ND

    adds = ["nd_tanf_countable_earned_income_person"]



from policyengine_us.model_api import *


class tob_revenue_total(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    label = "Total trust fund revenue from SS benefit taxation"
    documentation = (
        "Tax revenue from taxation of Social Security benefits using "
        "branching methodology per 42 U.S.C. 401 note."
    )
    unit = USD
    reference = "https://www.law.cornell.edu/uscode/text/42/401"

    def formula(tax_unit, period, parameters):
        """
        Calculate total trust fund revenue using branching + neutralization.

        This isolates the tax impact of including taxable SS in AGI by
        comparing tax with and without taxable SS.
        """
        sim = tax_unit.simulation

        # Calculate income tax WITH taxable SS
        income_tax_with_ss = tax_unit("income_tax", period)

        # Create branch and neutralize taxable SS
        branch = sim.get_branch("tob_calc", clone_system=True)
        branch.tax_benefit_system.neutralize_variable(
            "tax_unit_taxable_social_security"
        )

        # Delete non-input cached variables to force recalculation
        input_vars = set(branch.input_variables)
        for var_name in list(branch.tax_benefit_system.variables.keys()):
            if var_name not in input_vars:
                try:
                    branch.delete_arrays(var_name)
                except:
                    pass

        # Recalculate income tax without taxable SS
        income_tax_without_ss = branch.tax_unit("income_tax", period)

        # Clean up branch
        del sim.branches["tob_calc"]

        return income_tax_with_ss - income_tax_without_ss



from policyengine_us.model_api import *


class mi_fip_resources_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Michigan FIP based on resources"
    definition_period = MONTH
    reference = (
        "https://mdhhs-pres-prod.michigan.gov/olmweb/EX/BP/Public/BEM/400.pdf",
        "https://www.michigan.gov/mdhhs/-/media/Project/Websites/mdhhs/Inside-MDHHS/Reports-and-Statistics---Human-Services/State-Plans-and-Federal-Regulations/TANF_State_Plan_2023.pdf",
    )
    defined_for = StateCode.MI

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.mi.mdhhs.fip

        # Get total household assets
        # Using spm_unit_assets as a proxy for countable assets
        # Full implementation would apply specific asset exclusions per BEM 400
        # Note: spm_unit_assets is a YEAR variable, so we access it with this_year
        assets = spm_unit("spm_unit_assets", period.this_year)

        return assets <= p.resources.limit



from policyengine_us.model_api import *


class va_tanf_countable_unearned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "VA TANF countable unearned income"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.VA
    reference = "https://www.dss.virginia.gov/files/division/bp/tanf/manual/300_11-20.pdf#page=58"

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.va.dss.tanf.income.deductions
        gross_unearned = add(spm_unit, period, ["tanf_gross_unearned_income"])
        # Apply child support disregard
        child_support = add(spm_unit, period, ["child_support_received"])
        child_support_disregard = min_(child_support, p.child_support)
        return max_(gross_unearned - child_support_disregard, 0)



from policyengine_us.model_api import *


class pa_tanf_earned_income_after_deductions_person(Variable):
    value_type = float
    entity = Person
    label = "Pennsylvania TANF earned income after deductions per person"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.PA
    reference = (
        "http://services.dpw.state.pa.us/oimpolicymanuals/cash/160_Income_Deductions/160_2_TANF_Earned_Income_Deductions.htm",
        "https://www.law.cornell.edu/regulations/pennsylvania/55-Pa-Code-SS-183-94",
    )

    def formula(person, period, parameters):
        p = parameters(period).gov.states.pa.dhs.tanf.income.deductions

        gross_earned = person("tanf_gross_earned_income", period)
        disregard_eligible = person.spm_unit(
            "pa_tanf_disregard_eligible", period
        )

        after_eid = gross_earned * (1 - p.earned_income_disregard.percentage)
        after_wed = max_(after_eid - p.work_expense.additional, 0)

        return where(disregard_eligible, after_wed, gross_earned)



from policyengine_us.model_api import *


class wi_works_countable_resources(Variable):
    value_type = float
    entity = SPMUnit
    label = "Wisconsin Works countable resources"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://dcf.wisconsin.gov/manuals/w-2-manual/Production/03/03.3.4_COUNTING_ASSETS.htm",
        "https://docs.legis.wisconsin.gov/code/admin_code/dcf/101_199/101/09/3/b",
    )
    defined_for = StateCode.WI

    def formula(spm_unit, period, parameters):
        liquid_assets = spm_unit("spm_unit_cash_assets", period)
        vehicle_value = spm_unit.household("household_vehicles_value", period)
        p = parameters(period).gov.states.wi.dcf.works.asset
        countable_vehicle = max_(vehicle_value - p.vehicle_exclusion, 0)
        return liquid_assets + countable_vehicle



from policyengine_us.model_api import *


class in_tanf_resources_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Indiana TANF resources eligible"
    definition_period = MONTH
    defined_for = StateCode.IN
    reference = (
        "https://iar.iga.in.gov/code/2026/470/10.3#470-10.3-4-2",
        "https://iga.in.gov/laws/2025/ic/titles/12/#12-14-1-1",
    )

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states["in"].fssa.tanf.resources.limit
        countable = spm_unit("in_tanf_countable_resources", period)
        is_enrolled = spm_unit("is_tanf_enrolled", period)

        limit = where(
            is_enrolled,
            p.while_receiving.amount,
            p.at_application.amount,
        )
        return countable <= limit



from policyengine_us.model_api import *


class sd_tanf_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Income eligible for South Dakota TANF"
    definition_period = MONTH
    reference = "https://sdlegislature.gov/Rules/Administrative/67:10:05:03"
    defined_for = StateCode.SD

    def formula(spm_unit, period, parameters):
        countable_income = spm_unit("sd_tanf_countable_income", period)
        payment_standard = spm_unit("sd_tanf_payment_standard", period)
        return countable_income <= payment_standard



from policyengine_us.model_api import *


class nd_tanf(Variable):
    value_type = float
    entity = SPMUnit
    label = "North Dakota Temporary Assistance for Needy Families"
    unit = USD
    definition_period = MONTH
    reference = "https://www.nd.gov/dhs/policymanuals/40019/400_19_110_20.htm"
    defined_for = "nd_tanf_eligible"

    def formula(spm_unit, period, parameters):
        standard_of_need = spm_unit("nd_tanf_standard_of_need", period)
        countable_income = spm_unit("nd_tanf_countable_income", period)
        return max_(standard_of_need - countable_income, 0)



from policyengine_us.model_api import *


class ut_fep_payment_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Utah Family Employment Program payment standard"
    unit = USD
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/utah/Utah-Admin-Code-R986-200-239"
    defined_for = StateCode.UT

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ut.dwf.fep.payment_standard
        size = spm_unit("spm_unit_size", period)
        size_capped = min_(size, p.max_unit_size)
        return p.amount[size_capped]



from policyengine_us.model_api import *


class nv_tanf_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Nevada TANF countable earned income"
    unit = USD
    definition_period = MONTH
    reference = "https://dss.nv.gov/uploadedFiles/dwssnvgov/content/TANF/TANF_State_Plan_FINAL%20_Effective_12.31.20.pdf#page=17"
    defined_for = StateCode.NV
    # Sum person-level countable earned income.
    # The work expense deduction ($90 or 20%, whichever is greater)
    # is applied per person, then summed to the household.
    adds = ["nv_tanf_earned_income_after_disregard"]



from policyengine_us.model_api import *


class tn_ff(Variable):
    value_type = float
    entity = SPMUnit
    label = "Tennessee Families First"
    unit = USD
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/tennessee/Tenn-Comp-R-Regs-1240-01-50-.20"
    defined_for = "tn_ff_eligible"

    def formula(spm_unit, period, parameters):
        # Tennessee Families First TANF benefit using fill-the-gap methodology.
        # The benefit equals the minimum of: (1) the payment standard (SPA or DGPA),
        # or (2) the deficit (Consolidated Need Standard minus countable income).
        # No payment is made if the calculated benefit is less than the minimum grant.
        # Get payment standard (SPA or DGPA based on eligibility)
        payment_standard = spm_unit("tn_ff_payment_standard", period)

        # Calculate countable income
        countable_income = spm_unit("tn_ff_countable_income", period)

        p = parameters(period).gov.states.tn.dhs.ff.payment
        unit_size = spm_unit("spm_unit_size", period.this_year)
        capped_size = min_(unit_size, p.max_family_size)

        deficit = max_(
            p.consolidated_need_standard[capped_size] - countable_income, 0
        )
        calculated_benefit = min_(payment_standard, deficit)

        return where(
            calculated_benefit >= p.minimum_grant, calculated_benefit, 0
        )



from policyengine_us.model_api import *


class mo_tanf_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = (
        "Eligible for Missouri Temporary Assistance for Needy Families (TANF)"
    )
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/missouri/13-CSR-40-2-310",
        "https://dssmanuals.mo.gov/temporary-assistance-case-management/0200-000-00/",
    )
    defined_for = StateCode.MO

    def formula(spm_unit, period, parameters):
        person = spm_unit.members
        demographic = person("is_person_demographic_tanf_eligible", period)
        has_eligible_member = spm_unit.any(demographic)
        income_eligible = spm_unit("mo_tanf_income_eligible", period)
        resources_eligible = spm_unit("mo_tanf_resources_eligible", period)
        return has_eligible_member & income_eligible & resources_eligible



from policyengine_us.model_api import *


class il_pi_has_developmental_delay(Variable):
    value_type = bool
    entity = SPMUnit
    label = "SPM unit has developmental delay or disability for Illinois PI"
    definition_period = YEAR
    reference = "https://www.isbe.net/Documents/Prevention-Initiative-Eligibility-Form.pdf#page=2"
    defined_for = StateCode.IL

    def formula(spm_unit, period, parameters):
        # Factor 18: Member of household has developmental delay or disability
        has_disability = spm_unit.members("is_disabled", period)
        has_dev_delay = spm_unit.members("has_developmental_delay", period)
        return spm_unit.any(has_disability | has_dev_delay)



from policyengine_us.model_api import *


class ms_tanf_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Mississippi TANF countable income"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.MS
    reference = (
        "https://www.mdhs.ms.gov/wp-content/uploads/2018/02/MDHS_TANF-Eligibility-Flyer.pdf",
        "https://www.law.cornell.edu/regulations/mississippi/Miss-Code-tit-18-pt-19",
    )
    # NOTE: Mississippi has 6-month and 3-month total earned income disregards
    # that cannot be tracked in PolicyEngine. This simplified implementation
    # uses gross income without those time-limited disregards.

    adds = ["tanf_gross_earned_income", "tanf_gross_unearned_income"]



from policyengine_us.model_api import *


class il_hbi_resource_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Meets Illinois HBI resource eligibility"
    definition_period = YEAR
    defined_for = StateCode.IL
    reference = ("https://www.dhs.state.il.us/page.aspx?item=161600",)
    # Illinois HBIS (Health Benefits for Immigrant Seniors) has a resource limit
    # of $17,500 or less in non-exempt resources per household.
    #
    # This resource test only applies to seniors (age 65+) in the HBIS program.
    # Children (All Kids) and adults (HBIA) do not have a resource test.

    def formula(person, period, parameters):
        p = parameters(period).gov.states.il.hfs.hbi.eligibility

        age = person("age", period)
        is_senior = age >= p.senior.min_age

        # Get household assets
        household_assets = person.spm_unit("spm_unit_assets", period)

        # Seniors must meet the resource test
        # Non-seniors automatically pass (no resource test)
        return where(
            is_senior,
            household_assets <= p.senior.resource_limit,
            True,
        )



from policyengine_us.model_api import *


class sd_tanf_is_shared_living(Variable):
    value_type = bool
    entity = SPMUnit
    label = "South Dakota TANF shared living arrangement"
    definition_period = MONTH
    reference = "https://sdlegislature.gov/Rules/Administrative/67:10:05:03"
    defined_for = StateCode.SD



from policyengine_us.model_api import *


class in_tanf_countable_income_for_payment(Variable):
    value_type = float
    entity = SPMUnit
    label = "Indiana TANF countable income for payment calculation"
    unit = USD
    definition_period = MONTH
    reference = "https://iar.iga.in.gov/code/2026/470/10.3#470-10.3-4-4"
    defined_for = StateCode.IN

    adds = ["in_tanf_countable_earned_income", "tanf_gross_unearned_income"]



from policyengine_us.model_api import *


class ct_tfa_countable_income_at_application(Variable):
    value_type = float
    entity = SPMUnit
    label = "Connecticut Temporary Family Assistance (TFA) countable income at application"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.CT
    reference = "https://portal.ct.gov/dss/-/media/departments-and-agencies/dss/state-plans-and-federal-reports/tanf-state-plan/ct-tanf-state-plan-2024---2026---41524-amendment.pdf#page=10"

    adds = [
        "ct_tfa_countable_earned_income_at_application",
        "ct_tfa_countable_unearned_income",
    ]



from policyengine_us.model_api import *
from policyengine_us.variables.gov.hhs.hhs_smi import smi


class ca_child_care_smi(Variable):
    value_type = float
    entity = SPMUnit
    label = "California child care State Median Income"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.CA
    reference = "https://www.cde.ca.gov/sp/cd/ci/mb2505.asp"

    def formula(spm_unit, period, parameters):
        # California uses July 1 fiscal year for SMI
        year = period.start.year
        month = period.start.month
        if month >= 7:
            instant_str = f"{year}-07-01"
        else:
            instant_str = f"{year - 1}-07-01"

        size = spm_unit("spm_unit_size", period.this_year)
        state = spm_unit.household("state_code_str", period.this_year)

        return smi(size, state, instant_str, parameters) / MONTHS_IN_YEAR



from policyengine_us.model_api import *


class nh_fanf_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "New Hampshire FANF countable income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.dhhs.nh.gov/sr_htm/html/sr_97-03_dated_02_97.htm",
        "https://www.dhhs.nh.gov/fam_htm/newfam.htm",
    )
    defined_for = StateCode.NH

    adds = ["nh_fanf_countable_earned_income", "tanf_gross_unearned_income"]



from policyengine_us.model_api import *


class mo_agi_subtractions(Variable):
    value_type = float
    entity = Person
    label = "Missouri AGI subtractions"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.MO

    def formula(person, period, parameters):
        p = parameters(period).gov.states.mo.tax.income.subtractions
        total_subtractions = add(person, period, p.agi_subtractions)
        # Prevent negative subtractions from acting as additions
        return max_(0, total_subtractions)



from policyengine_us.model_api import *


class mn_mfip_dependent_care_deduction(Variable):
    value_type = float
    entity = SPMUnit
    label = "Minnesota MFIP dependent care deduction"
    unit = USD
    definition_period = MONTH
    reference = "https://www.dhs.state.mn.us/main/idcplg?IdcService=GET_DYNAMIC_CONVERSION&RevisionSelectionMethod=LatestReleased&dDocName=cm_001809"
    defined_for = StateCode.MN

    def formula(spm_unit, period, parameters):
        # Per MN Stat. 142G.16, Subd. 1(b)(2):
        # Deduct actual dependent care up to $200/child under 2, $175/child 2+.
        # Only applies to eligibility test, not benefit calculation.
        p = parameters(
            period
        ).gov.states.mn.dcyf.mfip.income.deductions.dependent_care
        person = spm_unit.members
        dependent = person("is_tax_unit_dependent", period)
        age = person("monthly_age", period)
        childcare_expenses = spm_unit("childcare_expenses", period)
        max_deduction_per_child = p.calc(age) * dependent
        total_max_deduction = spm_unit.sum(max_deduction_per_child)
        return min_(childcare_expenses, total_max_deduction)



from policyengine_us.model_api import *


class mi_fip_earned_income_after_deductions_for_benefit_person(Variable):
    value_type = float
    entity = Person
    label = (
        "Michigan FIP earned income after deductions for benefit per person"
    )
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.MI
    reference = (
        "https://mdhhs-pres-prod.michigan.gov/olmweb/ex/BP/Public/BEM/518.pdf#page=5",
        "https://mdhhs-pres-prod.michigan.gov/olmweb/EX/BP/Public/BEM/520.pdf#page=4",
    )

    def formula(person, period, parameters):
        p = parameters(period).gov.states.mi.mdhhs.fip.income.disregard

        # Get gross earned income for this person
        gross_earned = person("tanf_gross_earned_income", period)

        # BEM 518 Page 5 - "Issuance Earned Income Disregard"
        # BEM 520 Section D - Issuance Test (for benefit calculation)
        # "Deduct $200 from each person's countable earnings. Then deduct an
        # additional 50 percent of each person's remaining earnings."

        # This is used for BENEFIT CALCULATION for ALL recipients
        # (both new applicants and enrolled recipients use 50% for benefits)

        # Step 1: Deduct $200 from this person's earnings
        flat_deduction = p.flat
        remainder = max_(gross_earned - flat_deduction, 0)

        # Step 2: Deduct 50% of remainder (for benefit calculation)
        percent_deduction = remainder * p.ongoing_rate

        # Step 3: Countable income (cannot be negative)
        # "The total disregard cannot exceed countable earnings" - BEM 518
        return max_(gross_earned - flat_deduction - percent_deduction, 0)



from policyengine_us.model_api import *


class de_tanf_resources_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Delaware TANF resources eligible"
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/delaware/16-Del-Admin-Code-SS-4000-4002"
    defined_for = StateCode.DE

    def formula(spm_unit, period, parameters):
        # Per DSSM 4002: Resource limit is $10,000
        p = parameters(period).gov.states.de.dhss.tanf
        resources = spm_unit("spm_unit_assets", period.this_year)

        return resources <= p.resource_limit



from policyengine_us.model_api import *


class fl_tca_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Florida TCA eligible"
    definition_period = MONTH
    reference = (
        "https://www.leg.state.fl.us/statutes/index.cfm?App_mode=Display_Statute&URL=0400-0499/0414/Sections/0414.095.html",
        "https://www.myflfamilies.com/services/public-assistance/temporary-cash-assistance",
    )
    defined_for = StateCode.FL

    def formula(spm_unit, period, parameters):
        # Use federal demographic eligibility (minor child with deprived parent)
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)

        # Use federal immigration eligibility (at least one citizen or qualified immigrant)
        immigration_eligible = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )

        # Must pass both income tests
        gross_income_eligible = spm_unit(
            "fl_tca_gross_income_eligible", period
        )
        net_income_eligible = spm_unit("fl_tca_net_income_eligible", period)

        # Must pass resource test
        resource_eligible = spm_unit("fl_tca_resources_eligible", period)

        return (
            demographic_eligible
            & immigration_eligible
            & gross_income_eligible
            & net_income_eligible
            & resource_eligible
        )



from policyengine_us.model_api import *


class ks_tanf_resources_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Kansas TANF resource eligibility"
    definition_period = MONTH
    reference = (
        "https://content.dcf.ks.gov/ees/keesm/current/keesm5110.htm",
        "https://content.dcf.ks.gov/ees/keesm/current/keesm5000.htm",
    )
    defined_for = StateCode.KS

    def formula(spm_unit, period, parameters):
        # Per KEESM 5110: Resource limit is $3,000
        p = parameters(period).gov.states.ks.dcf.tanf.resource_limit
        resources = spm_unit("spm_unit_cash_assets", period.this_year)
        return resources <= p.amount



from policyengine_us.model_api import *


class ok_tanf(Variable):
    value_type = float
    entity = SPMUnit
    label = "Oklahoma TANF"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/oklahoma/OAC-340-10-3-59"
    )
    defined_for = "ok_tanf_eligible"

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ok.dhs.tanf

        payment_standard = spm_unit("ok_tanf_payment_standard", period)
        countable_income = spm_unit("ok_tanf_countable_income", period)

        # Per OAC 340:10-3-59: Benefit = Payment Standard - Countable Income
        benefit = max_(payment_standard - countable_income, 0)

        # Per OAC 340:10-3-59: Minimum benefit is $10; if less, no payment issues
        return where(benefit >= p.benefit.minimum_benefit, benefit, 0)



from policyengine_us.model_api import *


class nv_tanf_resources_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Nevada TANF resources eligible"
    definition_period = MONTH
    reference = "https://dss.nv.gov/TANF/TANF_FAQ-Eligibility_Criteria-R/S/"
    defined_for = StateCode.NV

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.nv.dwss.tanf.resource_limit
        # Nevada excludes two vehicles and home, but we use total assets
        # as a simplified approach
        assets = spm_unit("spm_unit_assets", period.this_year)
        return assets <= p.amount



from policyengine_us.model_api import *


class ga_tanf_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = (
        "Eligible for Georgia Temporary Assistance for Needy Families (TANF)"
    )
    definition_period = MONTH
    reference = (
        "https://rules.sos.ga.gov/gac/290-2-28",
        "https://dfcs.georgia.gov/services/temporary-assistance-needy-families/tanf-eligibility-requirements",
        "https://pamms.dhs.ga.gov/dfcs/tanf/",
    )
    defined_for = StateCode.GA

    def formula(spm_unit, period, parameters):
        person = spm_unit.members

        # Must meet demographic requirements (minor child OR pregnant woman)
        # Use federal demographic eligibility directly
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)

        # Must have at least one citizen or legal immigrant
        # Use federal immigration eligibility
        immigration_status_eligible = spm_unit.any(
            person("is_citizen_or_legal_immigrant", period)
        )

        # Must meet income requirements (gross and net income tests)
        income_eligible = spm_unit("ga_tanf_income_eligible", period)

        # Must meet resource requirements (assets <= $1,000)
        resources_eligible = spm_unit("ga_tanf_resources_eligible", period)

        # All requirements must be met
        return (
            demographic_eligible
            & immigration_status_eligible
            & income_eligible
            & resources_eligible
        )



from policyengine_us.model_api import *


class il_pi_is_poverty(Variable):
    value_type = bool
    entity = Person
    label = "Experiencing poverty for Illinois PI (Factor 36, 10 points)"
    definition_period = YEAR
    reference = (
        "https://www.isbe.net/Documents/Prevention-Initiative-Eligibility-Form.pdf#page=3",
    )
    defined_for = StateCode.IL

    def formula(person, period, parameters):
        spm_unit = person.spm_unit
        # Factor 36 (10 pts): Child/Family experiencing poverty (200% FPL).
        p = parameters(period).gov.states.il.isbe.pi.eligibility.income
        fpg = spm_unit("spm_unit_fpg", period)
        income = spm_unit("il_isbe_countable_income", period)
        threshold = fpg * p.poverty_rate
        return income <= threshold



from policyengine_us.model_api import *


class medicare_quarters_of_coverage(Variable):
    value_type = int
    entity = Person
    label = "Medicare quarters of coverage"
    documentation = (
        "Number of quarters of Social Security-covered employment. "
        "40 quarters required for premium-free Part A."
    )
    definition_period = YEAR



from policyengine_us.model_api import *


class hi_tanf_gross_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Hawaii TANF gross income eligibility"
    definition_period = MONTH
    reference = (
        "https://humanservices.hawaii.gov/wp-content/uploads/2024/12/Hawaii_TANF_State_Plan_Signed_Certified-Eff_20231001.pdf#page=19",
    )
    defined_for = StateCode.HI

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.hi.dhs.tanf

        gross_income = spm_unit("hi_tanf_gross_income", period)

        # Gross income limit = 185% of Standard of Need
        unit_size = spm_unit("spm_unit_size", period)
        capped_size = min_(unit_size, p.max_unit_size)
        standard_of_need = p.standard_of_need.amount[capped_size]
        gross_income_limit = (
            standard_of_need * p.eligibility.gross_income_limit_rate
        )

        return gross_income <= gross_income_limit



from policyengine_us.model_api import *


class ar_tea(Variable):
    value_type = float
    entity = SPMUnit
    label = "Arkansas Transitional Employment Assistance"
    unit = USD
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/arkansas/208-00-13-Ark-Code-R-SS-001"
    defined_for = "ar_tea_eligible"

    def formula(spm_unit, period, parameters):
        # Per TEA Manual Section 2362 - Reduced Payment - Gross Income Trigger
        p = parameters(period).gov.states.ar.dhs.tea

        maximum_benefit = spm_unit("ar_tea_maximum_benefit", period)
        countable_income = spm_unit("ar_tea_countable_income", period)

        # Check if gross income triggers 50% payment reduction
        gross_income = add(
            spm_unit,
            period,
            ["tanf_gross_earned_income", "tanf_gross_unearned_income"],
        )

        above_trigger = gross_income >= p.payment_standard.trigger.amount
        reduced_payment = maximum_benefit * (
            1 - p.payment_standard.trigger.reduction_rate
        )

        # When gross income >= trigger: payment is 50% of max (no subtraction)
        # When gross income < trigger: payment is max - countable income
        return where(
            above_trigger,
            reduced_payment,
            max_(maximum_benefit - countable_income, 0),
        )



from policyengine_us.model_api import *


class mn_mfip_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Minnesota MFIP due to income"
    definition_period = MONTH
    reference = (
        "https://www.revisor.mn.gov/statutes/cite/142G.02#stat.142G.02.42"
    )
    defined_for = StateCode.MN

    def formula(spm_unit, period, parameters):
        # Per MN Stat. 142G.16, Subd. 1:
        # Countable income (with dependent care deduction) < Family Wage Level
        countable = spm_unit(
            "mn_mfip_countable_income_for_eligibility", period
        )
        family_wage_level = spm_unit("mn_mfip_family_wage_level", period)
        return countable < family_wage_level



from policyengine_us.model_api import *


class ak_atap_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Alaska ATAP countable earned income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/alaska/7-AAC-45.480",
        "https://www.law.cornell.edu/regulations/alaska/7-AAC-45.485",
    )
    defined_for = StateCode.AK

    def formula(spm_unit, period, parameters):
        # Per 7 AAC 45.480: Work incentive deduction applies per person
        # Per 7 AAC 45.485: Childcare deduction applies at household level
        person_countable = add(
            spm_unit, period, ["ak_atap_countable_earned_income_person"]
        )
        childcare_deduction = spm_unit("ak_atap_childcare_deduction", period)
        return max_(person_countable - childcare_deduction, 0)



from policyengine_us.model_api import *


class pa_tanf_disregard_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Pennsylvania TANF earned income disregard eligibility"
    definition_period = MONTH
    defined_for = StateCode.PA
    reference = "https://www.law.cornell.edu/regulations/pennsylvania/55-Pa-Code-SS-183-94"

    def formula(spm_unit, period, parameters):
        person = spm_unit.members
        p = parameters(period).gov.states.pa.dhs.tanf.income.deductions

        is_enrolled = spm_unit("is_tanf_enrolled", period)

        gross_earned = person("tanf_gross_earned_income", period)
        preliminary_earned_per_person = max_(
            gross_earned - p.work_expense.initial, 0
        )
        preliminary_earned = spm_unit.sum(preliminary_earned_per_person)

        preliminary_unearned = add(
            spm_unit, period, ["tanf_gross_unearned_income"]
        )

        preliminary_income = preliminary_earned + preliminary_unearned
        standard_of_need = spm_unit("pa_tanf_standard_of_need", period)

        return is_enrolled | (preliminary_income < standard_of_need)



from policyengine_us.model_api import *


class pa_tanf_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Pennsylvania TANF income eligibility"
    definition_period = MONTH
    defined_for = StateCode.PA
    reference = "https://www.pacodeandbulletin.gov/Display/pacode?file=/secure/pacode/data/055/chapter183/chap183toc.html"

    def formula(spm_unit, period, parameters):
        countable_income = spm_unit("pa_tanf_countable_income", period)
        maximum_benefit = spm_unit("pa_tanf_maximum_benefit", period)

        return countable_income < maximum_benefit



from policyengine_us.model_api import *


class mi_fip_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Michigan Family Independence Program"
    definition_period = MONTH
    reference = (
        "https://www.michigan.gov/mdhhs/-/media/Project/Websites/mdhhs/Inside-MDHHS/Reports-and-Statistics---Human-Services/State-Plans-and-Federal-Regulations/TANF_State_Plan_2023.pdf",
        "https://mdhhs-pres-prod.michigan.gov/olmweb/ex/bp/public/bem/100.pdf",
    )
    defined_for = StateCode.MI

    def formula(spm_unit, period, parameters):
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        immigration_eligible = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )
        income_eligible = spm_unit("mi_fip_income_eligible", period)
        resources_eligible = spm_unit("mi_fip_resources_eligible", period)

        # NOTE: Time limits and work requirements not modeled

        return (
            demographic_eligible
            & immigration_eligible
            & income_eligible
            & resources_eligible
        )



from policyengine_us.model_api import *


class hi_tanf_maximum_benefit(Variable):
    value_type = float
    entity = SPMUnit
    label = "Hawaii TANF maximum benefit"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://humanservices.hawaii.gov/wp-content/uploads/2024/12/Hawaii_TANF_State_Plan_Signed_Certified-Eff_20231001.pdf#page=22",
    )
    defined_for = StateCode.HI

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.hi.dhs.tanf

        unit_size = spm_unit("spm_unit_size", period)
        capped_size = min_(unit_size, p.max_unit_size)

        # Get Standard of Need for household size
        son = p.standard_of_need.amount[capped_size]

        # Standard of Assistance = SON × 48%
        # NOTE: SOA is reduced by 20% after the family receives their initial
        # two full months of benefits for mandatory work-required households.
        # This reduction cannot be modeled as PolicyEngine cannot track
        # cumulative months of benefit receipt.
        return son * p.standard_of_assistance.rate



from .ctc import create_ri_ctc_reform
from .exemption import create_ri_exemption_reform_fn
from .high_earner_tax import create_ri_high_earner_tax_reform
from .social_security_exemption import (
    create_ri_social_security_exemption_reform,
)



from policyengine_us.model_api import *


class in_tanf_payment_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Indiana TANF payment eligible"
    definition_period = MONTH
    reference = (
        "https://iar.iga.in.gov/code/2026/470/10.3#470-10.3-4-3",
        "https://iga.in.gov/laws/2025/ic/titles/12/#12-14-2-5",
    )
    defined_for = StateCode.IN

    def formula(spm_unit, period, parameters):
        countable = spm_unit("in_tanf_countable_income_for_payment", period)
        payment_standard = spm_unit("in_tanf_payment_standard", period)
        return countable < payment_standard



from policyengine_us.model_api import *


class ok_tanf_dependent_care_deduction(Variable):
    value_type = float
    entity = SPMUnit
    label = "Oklahoma TANF dependent care deduction"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/oklahoma/OAC-340-10-3-33"
    )
    defined_for = StateCode.OK

    def formula(spm_unit, period, parameters):
        # Per OAC 340:10-3-33(b): Dependent care expenses may be deducted
        # up to $200/month for dependents under age 2
        # up to $175/month for dependents age 2 and older
        p = parameters(period).gov.states.ok.dhs.tanf.income
        person = spm_unit.members

        # Get dependent status and age in years
        dependent = person("is_tax_unit_dependent", period)
        age = person("monthly_age", period)  # Returns true age in years

        # Calculate maximum deduction per dependent based on age
        max_deduction_per_dependent = p.deductions.dependent_care.calc(age)
        total_max_deduction = spm_unit.sum(
            max_deduction_per_dependent * dependent
        )

        # Cap at actual childcare expenses
        childcare_expenses = spm_unit("childcare_expenses", period)

        return min_(childcare_expenses, total_max_deduction)



from policyengine_us.model_api import *


class mi_fip_countable_income_for_benefit(Variable):
    value_type = float
    entity = SPMUnit
    label = "Michigan FIP countable income for benefit"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://mdhhs-pres-prod.michigan.gov/olmweb/ex/BP/Public/BEM/520.pdf",
        "https://mdhhs-pres-prod.michigan.gov/olmweb/ex/BP/Public/BEM/503.pdf",
    )
    defined_for = StateCode.MI

    # BEM 520 Section D Line 9: "Enter the sum of line 7 and line 8"
    # Line 7: Countable earned income (after per-person deductions)
    # Line 8: All countable unearned income (gross, no deductions per BEM 503)
    adds = [
        "mi_fip_countable_earned_income_for_benefit",
        "tanf_gross_unearned_income",
    ]



from policyengine_us.model_api import *


class ct_tfa(Variable):
    value_type = float
    entity = SPMUnit
    label = "Connecticut Temporary Family Assistance (TFA) benefit amount"
    unit = USD
    definition_period = MONTH
    reference = "https://portal.ct.gov/dss/-/media/departments-and-agencies/dss/state-plans-and-federal-reports/tanf-state-plan/ct-tanf-state-plan-2024---2026---41524-amendment.pdf#page=10"
    defined_for = "ct_tfa_eligible"

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ct.dss.tfa.payment.high_earnings
        # Unearned income are counted dollar for dollar against TFA benefit amount
        # https://portal.ct.gov/dss/knowledge-base/articles/cash-assistance/temporary-family-assistance?language=en_US
        payment_standard = spm_unit("ct_tfa_payment_standard", period)
        countable_unearned_income = spm_unit(
            "ct_tfa_countable_unearned_income", period
        )
        benefit_amount = max_(payment_standard - countable_unearned_income, 0)
        # When gross earnings are >= 171% of FPG, reduce the benefit by 20%
        gross_earnings = add(spm_unit, period, ["tanf_gross_earned_income"])
        fpg = spm_unit("tanf_fpg", period)
        high_income_threshold = p.rate * fpg
        high_income = gross_earnings >= high_income_threshold

        reduction_multiplier = 1 - p.reduction_rate

        return where(
            high_income,
            benefit_amount * reduction_multiplier,
            benefit_amount,
        )



from policyengine_us.model_api import *


class de_tanf_countable_unearned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Delaware TANF countable unearned income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/delaware/16-Del-Admin-Code-SS-4000-4008",
        "https://dhss.delaware.gov/wp-content/uploads/sites/11/dss/pdf/detanfstateplan2017.pdf#page=6",
    )
    defined_for = StateCode.DE

    def formula(spm_unit, period, parameters):
        # Gross unearned income minus $50 child support disregard
        p = parameters(period).gov.states.de.dhss.tanf.income
        gross_unearned = add(spm_unit, period, ["tanf_gross_unearned_income"])
        child_support = add(spm_unit, period, ["child_support_received"])
        child_support_disregard = min_(
            child_support, p.deductions.child_support
        )
        return max_(gross_unearned - child_support_disregard, 0)



from policyengine_us.model_api import *


class ok_tanf_work_expense_person(Variable):
    value_type = float
    entity = Person
    label = "Oklahoma TANF work expense deduction per person"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/oklahoma/OAC-340-10-3-33"
    )
    defined_for = StateCode.OK

    def formula(person, period, parameters):
        # Per OAC 340:10-3-33(a): Work expense based on weekly hours
        # $120 for <30 hours/week, $240 for 30+ hours/week
        p = parameters(period).gov.states.ok.dhs.tanf.income
        weekly_hours = person("weekly_hours_worked", period.this_year)

        # Only apply work expense if person has earnings
        has_earnings = person("tanf_gross_earned_income", period) > 0

        work_expense = p.deductions.work_expense.calc(weekly_hours)
        return where(has_earnings, work_expense, 0)



from policyengine_us.model_api import *


class sd_tanf_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for South Dakota TANF"
    definition_period = MONTH
    reference = "https://sdlegislature.gov/Rules/Administrative/67:10:01:05"
    defined_for = StateCode.SD

    def formula(spm_unit, period, parameters):
        # Use federal demographic eligibility (minor child or pregnant)
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        # Per ARSD 67:10:01:05: Must be US citizen or qualified alien
        immigration_eligible = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )
        income_eligible = spm_unit("sd_tanf_income_eligible", period)
        resources_eligible = spm_unit("sd_tanf_resources_eligible", period)

        return (
            demographic_eligible
            & immigration_eligible
            & income_eligible
            & resources_eligible
        )



from policyengine_us.model_api import *


class oh_owf_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Ohio OWF income eligibility"
    definition_period = MONTH
    defined_for = StateCode.OH
    reference = (
        "https://codes.ohio.gov/ohio-administrative-code/rule-5101:1-23-20"
    )

    def formula(spm_unit, period, parameters):
        # Check enrollment status
        enrolled = spm_unit("is_tanf_enrolled", period)

        # Initial eligibility test: gross income < 50% FPL (new applicants)
        initial_eligible = spm_unit("oh_owf_initial_income_eligible", period)

        # Ongoing eligibility test: countable income < payment standard
        countable_income = spm_unit("oh_owf_countable_income", period)
        payment_standard = spm_unit("oh_owf_payment_standard", period)
        ongoing_eligible = countable_income < payment_standard

        # Per ORC 5107.10(D):
        # - New applicants: must pass both initial (50% FPL) and ongoing tests
        # - Enrolled recipients: only need to pass ongoing test
        return where(
            enrolled,
            ongoing_eligible,
            initial_eligible & ongoing_eligible,
        )



from policyengine_us.model_api import *
from policyengine_core.periods import period as period_
from policyengine_core.periods import instant


def create_ri_high_earner_tax() -> Reform:
    class ri_income_tax_before_non_refundable_credits(Variable):
        value_type = float
        entity = TaxUnit
        label = "Rhode Island income tax before refundable credits"
        defined_for = StateCode.RI
        unit = USD
        definition_period = YEAR

        def formula(tax_unit, period, parameters):
            income = tax_unit("ri_taxable_income", period)
            p_baseline = parameters(period).gov.states.ri.tax.income.rate
            p_reform = parameters(period).gov.contrib.states.ri.high_earner_tax

            # Use reform brackets if in effect, otherwise use baseline
            reform_active = p_reform.in_effect
            reform_tax = p_reform.brackets.calc(income)
            baseline_tax = p_baseline.calc(income)

            return where(reform_active, reform_tax, baseline_tax)

    class reform(Reform):
        def apply(self):
            self.update_variable(ri_income_tax_before_non_refundable_credits)

    return reform


def create_ri_high_earner_tax_reform(parameters, period, bypass: bool = False):
    if bypass:
        return create_ri_high_earner_tax()

    p = parameters.gov.contrib.states.ri.high_earner_tax

    reform_active = False
    current_period = period_(period)

    for i in range(5):
        if p(current_period).in_effect:
            reform_active = True
            break
        current_period = current_period.offset(1, "year")

    if reform_active:
        return create_ri_high_earner_tax()
    else:
        return None


ri_high_earner_tax = create_ri_high_earner_tax_reform(None, None, bypass=True)



from policyengine_us.model_api import *


class il_aabd_use_reported_ssi(Variable):
    value_type = bool
    entity = Person
    definition_period = YEAR
    default_value = False
    label = "Use reported SSI for IL AABD calculation"
    documentation = (
        "When True, IL AABD uses ssi_reported instead of calculated ssi "
        "as unearned income. This allows API partners to show both federal "
        "SSI entitlement and IL AABD eligibility in a single response."
    )



from policyengine_us.model_api import *


class ne_adc_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Nebraska ADC income eligible"
    definition_period = MONTH
    reference = (
        "https://nebraskalegislature.gov/laws/statutes.php?statute=68-1726",
        "https://dhhs.ne.gov/Pages/Title-468.aspx",
    )
    defined_for = StateCode.NE

    def formula(spm_unit, period, parameters):
        # Per 468 NAC: countable income must be below standard of need
        countable_income = spm_unit("ne_adc_countable_income", period)
        need_standard = spm_unit("ne_adc_need_standard", period)
        return countable_income < need_standard



from policyengine_us.model_api import *


class msp_income_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Medicare Savings Program income eligible"
    definition_period = MONTH
    reference = (
        "https://www.medicare.gov/basics/costs/help/medicare-savings-programs",
        "https://www.law.cornell.edu/cfr/text/42/435.121",
    )

    def formula(person, period, parameters):
        # Income eligible if under the QI threshold (135% FPL)
        # which is the highest threshold for standard MSP levels
        p = parameters(
            period
        ).gov.hhs.medicare.savings_programs.eligibility.income.qi
        fpg = person("msp_fpg", period)
        countable_income = person("msp_countable_income", period)

        # Use QI threshold (135% FPL) as the outer bound
        qi_income_limit = fpg * p.fpl_limit

        return countable_income <= qi_income_limit



from policyengine_us.model_api import *


class or_tanf_adjusted_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Oregon TANF adjusted income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://oregon.public.law/rules/oar_461-001-0000",
        "https://oregon.public.law/rules/oar_461-160-0160",
    )
    defined_for = StateCode.OR

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states["or"].odhs.tanf.income
        countable_income = spm_unit("or_tanf_countable_income", period)
        gross_earned = add(spm_unit, period, ["tanf_gross_earned_income"])
        earned_income_disregard = gross_earned * p.earned_income_disregard_rate
        return max_(countable_income - earned_income_disregard, 0)



from policyengine_us.model_api import *


class me_tanf_maximum_benefit(Variable):
    value_type = float
    entity = SPMUnit
    label = "Maine TANF maximum benefit"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.mainelegislature.org/legis/statutes/22/title22sec3762.html",
        "https://www.law.cornell.edu/regulations/maine/10-144-C-M-R-ch-331-app-Charts",
    )
    defined_for = StateCode.ME

    def formula(spm_unit, period, parameters):
        # Per 10-144 C.M.R. Chapter 331, Appendix Charts (Table 2)
        p = parameters(period).gov.states.me.dhhs.tanf

        unit_size = spm_unit("spm_unit_size", period.this_year)

        # Cap at max unit size, then add incremental for larger households
        max_size = p.max_unit_size
        capped_size = min_(unit_size, max_size)
        additional_members = max_(unit_size - max_size, 0)

        # Look up base amount
        base_amount = p.maximum_benefit.amount[capped_size]

        # Add incremental amount for households larger than max size
        incremental = additional_members * p.maximum_benefit.each_additional

        return base_amount + incremental



from policyengine_us.model_api import *


class tn_ff_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Tennessee Families First countable income"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.TN
    reference = "https://publications.tnsosfiles.com/rules/1240/1240-01/1240-01-50.20081124.pdf#page=19"
    adds = [
        "tn_ff_countable_earned_income",
        "tanf_gross_unearned_income",
    ]



from policyengine_us.model_api import *


class msp_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Medicare Savings Program eligible"
    definition_period = MONTH
    reference = (
        "https://www.medicare.gov/basics/costs/help/medicare-savings-programs",
        "https://www.law.cornell.edu/cfr/text/42/435.121",
    )

    def formula(person, period, parameters):
        # Must be Medicare eligible
        medicare_eligible = person("is_medicare_eligible", period.this_year)
        income_eligible = person("msp_income_eligible", period)
        asset_eligible = person("msp_asset_eligible", period)

        return medicare_eligible & income_eligible & asset_eligible



from policyengine_us.model_api import *


class il_pfae_secondary_priority_factor_count(Variable):
    value_type = int
    entity = Person
    label = "Number of secondary priority factors for Illinois PFAE"
    definition_period = YEAR
    reference = (
        "https://www.isbe.net/pages/preschool-for-all.aspx",
        "https://www.isbe.net/Documents/pdg-eg-grant-enrollment-form.pdf#page=2",
    )
    defined_for = StateCode.IL

    def formula(person, period, parameters):
        spm_unit = person.spm_unit
        household = person.household

        # Factor 5: Income <= 100% FPL
        is_low_income = person("il_pfae_is_low_income", period)

        # Factor 6: Parent didn't complete high school (Household-level variable)
        parent_low_education = household(
            "parent_has_less_than_high_school_education", period
        )

        # Factor 7: Teen parent at birth of first child
        # Parent was under 20 when first child was born
        was_teen_parent = spm_unit(
            "il_isbe_was_teen_parent_at_first_birth", period
        )

        # Factor 8: Child or parent born outside US
        born_outside_us = person("is_born_outside_us", period)
        is_immigrant_family = spm_unit.any(born_outside_us)

        # Factor 9: Non-English speaking home (Household-level variable)
        is_non_english_home = household("is_non_english_speaking_home", period)

        # Factor 10: Active Duty Military family
        is_active_duty_military = person("military_basic_pay", period) > 0
        is_military_family = spm_unit.any(is_active_duty_military)

        # Factor 11: Developmental delay (without IEP)
        has_developmental_delay = person("has_developmental_delay", period)
        has_iep = person("has_individualized_education_program", period)
        delay_without_iep = has_developmental_delay & ~has_iep

        # Factor 12: No prior formal early learning
        no_prior_learning = person(
            "has_no_prior_formal_early_learning", period
        )

        factors = [
            is_low_income,
            parent_low_education,
            was_teen_parent,
            is_immigrant_family,
            is_non_english_home,
            is_military_family,
            delay_without_iep,
            no_prior_learning,
        ]

        return sum([f.astype(int) for f in factors])
